@@ -2,7 +2,7 @@
  *                 Finite Element Method Object Library        *
  *           class p1c2d : declaration for p1c2d               *
  *                    simula+@metz.ensam.fr                    *
- *	             GNU/linux version 0.2.0	               *
+ *	             GNU/linux version 0.2.6	               *
  *            software under General Public License            *
  ***************************************************************
  * copyright  2003,2004,2005,2006 CREUSE Emmanuel
@@ -41,7 +41,7 @@
 	     copyright \htmlonly &#169; \endhtmlonly  2003,2004,2005,2006 SOUALEM Nadir \n
 	     copyright \htmlonly &#169; \endhtmlonly Laboratoire de Physique et Mcanique des Matriaux (LPMM - UMR 7554) \n
 	     copyright \htmlonly &#169; \endhtmlonly Laboratoire de Mathmatiques et ses Applications de Valenciennes (LAMAV - EA 4015)
-    \version 0.2.0
+    \version 0.2.6
     \date 2003-2006
     \bug none
     \warning none
@@ -86,30 +86,55 @@
 */
 using namespace std;
 
-
 //=============================
 template <class T> class p1c2d:public mesh_tri_2d<T>
 //=============================
 {
+  using mesh<triangle<T>,T>::alive;
+  using mesh<triangle<T>,T>::dimspace_;
+  using mesh<triangle<T>,T>::nbvertex_;
+  using mesh<triangle<T>,T>::nbelement_;
+  using mesh<triangle<T>,T>::vertices;
+  using mesh<triangle<T>,T>::element;
+
+  using mesh_tri_2d<T>::nbsegments_;
+  using mesh_tri_2d<T>::SetOfEdges_;
+  using mesh_tri_2d<T>::Type_of_edge_;
+  using mesh_tri_2d<T>::EdgesPerTriangle_;
+  using mesh_tri_2d<T>::NbTrianglesPerEdge_;
+  using mesh_tri_2d<T>::TrianglesPerEdge_;
+  using mesh_tri_2d<T>::TrianglesPerTriangle_;
+  using mesh_tri_2d<T>::NumberOfNeighboorOfTriangles_;
+  using mesh_tri_2d<T>::NumLocNodeInTriangle_;
+  using mesh_tri_2d<T>:: NormalDirection_;
+  using mesh_tri_2d<T>::hminT_;
+  using mesh_tri_2d<T>::hE_;
+  using mesh_tri_2d<T>::hminE_;
+  using mesh_tri_2d<T>::NormEdge_;
+  using mesh_tri_2d<T>::NormalUnitVector_;
+  using mesh_tri_2d<T>::NbMaxSegmentsPerNode_;
+  using mesh_tri_2d<T>::NbSegmentsPerNode_;
+
+
   private :
   int DofNumber_; // Number of DDL par element (3 for p1c2d)
   int GlobalDofNumber_; // Number of DDL for the whole mesh (nbvertex_ for p1c2d)
   matrix<T> RefMassMatrix_; // 3*3 Mass matrix on the reference element
   vector<T> BuildBloc(T (*f)(T,T),int,integration<T>) const; 
   public :
-  p1c2d(); // Default Constructor
-  p1c2d(const char*,const char*); // Constructor
-  p1c2d(const char*); // Constructor
-  p1c2d(const p1c2d<T>&); // Copy Constructor
-  p1c2d<T>& operator=(const p1c2d<T>&);//p1c2d affectation
-  int DofNumber()const{return (DofNumber_);};
+   p1c2d(); // Default Constructor
+   p1c2d(const char*,const char*); // Constructor
+   p1c2d(const char*); // Constructor
+   p1c2d(const p1c2d<T>&); // Copy Constructor
+   p1c2d<T>& operator=(const p1c2d<T>&);//p1c2d affectation
+   int DofNumber()const{return (DofNumber_);};
   int GlobalDofNumber()const{return (GlobalDofNumber_);};
   matrix<T> RefMassMatrix()const{return(RefMassMatrix_);};
   vector<T> BasisFunction(T,T)const; 
   matrix<T> DiffBasisFunction(T xref=0,T yref=0)const;
   virtual int operator ()(int,int) const;
-  friend  int operator == <T> (const p1c2d<T>&,const p1c2d<T>&);
-  friend  int operator != <T> (const p1c2d<T>&,const p1c2d<T>&);
+  template <class FT> friend  int operator ==  (const p1c2d<FT>&,const p1c2d<FT>&);
+  template <class FT> friend  int operator !=  (const p1c2d<FT>&,const p1c2d<FT>&);
   matrix<T> LocalMassMatrix(int)const; // 3*3 Mass matrix on a given element
   matrix<T> LocalStiffnessMatrix(int)const; // 3*3 Stiffness matrix on a given element
   vector<T> Fk(T,T, int) const; // Affine transformation from the reference element to the actual one
@@ -125,11 +150,17 @@ template <class T> class p1c2d:public mesh_tri_2d<T>
   T error_L2_loc (T (*u)(T,T),const vector<T>& uh,int NbIntPts,int num_element) const ; // Local L2 error
   T error_L2 (T (*u)(T,T),const vector<T>& uh,int NbIntPts) const ; // L2 error
   T error_semi_H1(T (*dudx)(T,T),T (*dudy)(T,T),const vector<T>& uh,int NbIntPts) const; // Semi H1 error
+  T error_semi_H1_loc2(T (*dudx)(T,T),T (*dudy)(T,T),const vector<T>& uh,int NbIntPts,int num_element) const; // Semi H1 error
   T Grad_Normal_Jump_L2(const vector<T>& uh, int num_element) const;
   T Laplacian_Residual_L2(T (*f)(T,T),int num_element) const;
   void refine_FE(vector<int>&); // refine the Finite Element
   void refine_FE_angle(vector<int>&,T); // refine the Finite Element
+  T RT_L2Norm(T (*f)(T,T),int num_element) const;
 };
+
+
+
+
 /*!
   \brief Default Constructor 
   \n \n
@@ -277,12 +308,12 @@ return number(element[num_element-1][j]);
    \return 1 if EF1 and  EF2 are equal, 0 else
   */
 //==========================================================================
-template <class T>
-int operator ==(const p1c2d<T>& EF1,const p1c2d<T>& EF2)
+template <class FT>
+int operator ==(const p1c2d<FT>& EF1,const p1c2d<FT>& EF2)
 //==========================================================================
 {
   int boolean=1;
-  boolean*=(mesh_tri_2d<T>(EF1)==mesh_tri_2d<T>(EF2));
+  boolean*=(mesh_tri_2d<FT>(EF1)==mesh_tri_2d<FT>(EF2));
   boolean*=(EF1.DofNumber_==EF2.DofNumber_);
   boolean*=(EF1.GlobalDofNumber_==EF2.GlobalDofNumber_);
   boolean*=(EF1.RefMassMatrix_==EF2.RefMassMatrix_);
@@ -319,7 +350,7 @@ return(RefMassMatrix_*element[num_element-1].AbsDet());
    \param num_element element's number
    \return local stifness  matrix of the element num_element
   */
-///====================================================
+//====================================================
 template <class T> 
 matrix<T> p1c2d<T>::LocalStiffnessMatrix(int num_element) const
 //====================================================
@@ -732,7 +763,63 @@ GlobalError+=LocalError*element[num_element-1].AbsDet();
 }
 return(sqrt(GlobalError));
 }
+//======================================================
+template <class T>
+T p1c2d<T>::error_semi_H1_loc2 (T (*dudx)(T,T),T (*dudy)(T,T),const vector<T>& uh,int NbIntPts,int num_element) const
+//======================================================
+{
+T GlobalError=0;
+T LocalError;
+T omegak, omegal, xk, xl;
 
+T duhdx,duhdy;
+T duhdxhat,duhdyhat;
+vector<T> Fxkl;
+
+integration<T> itg;
+itg.Gauss(NbIntPts,0.0,1.0);
+
+matrix<T> DiffBasis(2,DofNumber_);
+matrix<T> InvMk;
+
+LocalError=0;
+InvMk=element[num_element-1].InvMk();
+
+   for (int k=1; k<=itg.NbIntPts(); k++)
+
+
+    {
+      omegak=itg.weight(k);
+      xk=itg.point(k);
+
+      for (int l=1; l<=itg.NbIntPts(); l++)
+	{
+	  omegal=itg.weight(l);
+          xl=(1-xk)*itg.point(l);
+	  DiffBasis=DiffBasisFunction(xk,xl);
+
+
+	  duhdxhat=0.0;
+	  duhdyhat=0.0;
+
+	  for (int i=1; i<=DofNumber_; i++)
+	  {
+	  duhdxhat+=uh[number(element[num_element-1][i])]*DiffBasis(1,i);
+	  duhdyhat+=uh[number(element[num_element-1][i])]*DiffBasis(2,i);
+	  }
+
+	  duhdx=duhdxhat*InvMk(1,1)+duhdyhat*InvMk(2,1);
+	  duhdy=duhdxhat*InvMk(1,2)+duhdyhat*InvMk(2,2);
+
+	  Fxkl=Fk(xk,xl,num_element);
+	  LocalError += omegak*omegal*(1-xk)*
+          (pow((dudx(Fxkl[1],Fxkl[2])-duhdx),2)+
+	   pow((dudy(Fxkl[1],Fxkl[2])-duhdy),2));
+	 }
+    }
+
+return(LocalError*element[num_element-1].AbsDet());
+}
 //======================================================
 template <class T>
 T p1c2d<T>::Grad_Normal_Jump_L2(const vector<T>& uh,int num_element) const
@@ -781,17 +868,17 @@ for(int edge=1;edge<=3;edge++)
  duhdx2=duhdxhat2*InvMk2(1,1)+duhdyhat2*InvMk2(2,1);
  duhdy2=duhdxhat2*InvMk2(1,2)+duhdyhat2*InvMk2(2,2);
 
- LocalJump=powf(NormEdge_[jseg],3.0/2.0)*
+ LocalJump=NormEdge_[jseg]*sqrt(NormEdge_[jseg])*
            abs(normal[1]*(duhdx2-duhdx1)+normal[2]*(duhdy2-duhdy1));
  }
  else // segment frontalier
  {
-  LocalJump=powf(NormEdge_[jseg],3.0/2.0)*
+  LocalJump=NormEdge_[jseg]*sqrt(NormEdge_[jseg])*
            abs(normal[1]*duhdx1+normal[2]*duhdy1);
  }
-SumJumps+=LocalJump;
+SumJumps+=pow(LocalJump,2);
 }
-return(SumJumps);
+return(sqrt(SumJumps));
 }
 //======================================================
 template <class T>
@@ -827,7 +914,7 @@ template <class T>
 void p1c2d<T>::refine_FE(vector<int> & TABTRI)
 //=============================
 {
-  refine_mesh(TABTRI);
+  (*this).refine_mesh(TABTRI);
   GlobalDofNumber_=nbvertex_;
 }
 
@@ -836,7 +923,16 @@ template <class T>
 void p1c2d<T>::refine_FE_angle(vector<int> & TABTRI, T angle)
 //=============================
 {
-  refine_mesh_angle(TABTRI,angle);
-  GlobalDofNumber_=nbvertex_;
+ (*this).refine_mesh_angle(TABTRI,angle); 
+GlobalDofNumber_=nbvertex_;
 }
+
+//======================================================
+template <class T>
+T p1c2d<T>::RT_L2Norm(T (*f)(T,T),int num_element) const
+//======================================================
+{
+ return((*this).Laplacian_Residual_L2(f,num_element)/element[num_element-1].diam());
+}
+
 #endif
