@@ -2,10 +2,10 @@
  *                 Finite Element Method Object Library        *
  *           class p1c3d : declaration for p1c3d               *
  *                    simula+@metz.ensam.fr                    *
- *	             GNU/linux version 0.0.8	               *
+ *	             GNU/linux version 0.1.0	               *
  *            software under General Public License            *
  ***************************************************************
- * copyright  2004 CREUSE Emmanuel
+ * copyright  2004,2006 CREUSE Emmanuel
  * copyright  2004 PIRAUT Frdric
  * copyright  Laboratoire de Physique et Mcanique des Matriaux (LPMM - UMR 7554)
  * copyright  Laboratoire de Mathmatiques et ses Applications de Valenciennes (LAMAV - EA 4015)
@@ -37,12 +37,12 @@
     </FONT>
     \endhtmlonly
 
-    \authors copyright \htmlonly &#169; \endhtmlonly  2004 CREUSE Emmanuel \n
+    \authors copyright \htmlonly &#169; \endhtmlonly  2004,2006 CREUSE Emmanuel \n
 	     copyright \htmlonly &#169; \endhtmlonly  2004 PIRAUT Frdric \n
 	     copyright \htmlonly &#169; \endhtmlonly Laboratoire de Physique et Mcanique des Matriaux (LPMM - UMR 7554) \n
 	     copyright \htmlonly &#169; \endhtmlonly Laboratoire de Mathmatiques et ses Applications de Valenciennes (LAMAV - EA 4015)
-    \version 0.0.8
-    \date 2004
+    \version 0.1.0
+    \date 2004,2006
     \bug none
     \warning none
 */
@@ -76,9 +76,16 @@
 
 using namespace std;
 //=============================
-template <class T> class p1c3d:public mesh<tetrahedre,T>
+template <class T> class p1c3d:public mesh<tetrahedre<T>,T>
 //=============================
 {
+  using mesh<tetrahedre<T>,T>::alive;
+  using mesh<tetrahedre<T>,T>::dimspace_;
+  using mesh<tetrahedre<T>,T>::nbvertex_;
+  using mesh<tetrahedre<T>,T>::nbelement_;
+  using mesh<tetrahedre<T>,T>::vertices;
+  using mesh<tetrahedre<T>,T>::element;
+
   private :
   int DofNumber_;
   int GlobalDofNumber_;
@@ -107,8 +114,8 @@ template <class T> class p1c3d:public mesh<tetrahedre,T>
   vector<T> BasisFunction(T,T,T)const;
   matrix<T> DiffBasisFunction(T xref=0,T yref=0,T zref=0)const;
   virtual int operator ()(int,int) const;
-  friend  int operator == <T> (const p1c3d<T>&,const p1c3d<T>&);
-  friend  int operator != <T> (const p1c3d<T>&,const p1c3d<T>&);
+  template <class FT> friend  int operator == (const p1c3d<FT>&,const p1c3d<FT>&);
+  template <class FT> friend  int operator != (const p1c3d<FT>&,const p1c3d<FT>&);
   matrix<T> LocalMassMatrix(int)const;
   matrix<T> LocalStiffnessMatrix(int)const;
   vector<T> Fk(T,T,T, int) const;
@@ -126,7 +133,7 @@ template <class T> class p1c3d:public mesh<tetrahedre,T>
 */  
 //============================================================================
 template <class T> 
-p1c3d<T>::p1c3d() :mesh<tetrahedre,T>()
+p1c3d<T>::p1c3d() :mesh<tetrahedre<T>,T>()
 //============================================================================
 {
  DofNumber_=0;
@@ -144,7 +151,7 @@ p1c3d<T>::p1c3d() :mesh<tetrahedre,T>()
 */
 //============================================================================
 template <class T> 
-p1c3d<T>::p1c3d(const char*file1,const char*file2) :mesh<tetrahedre,T>(file1,file2)
+p1c3d<T>::p1c3d(const char*file1,const char*file2) :mesh<tetrahedre<T>,T>(file1,file2)
 //============================================================================
 {
  DofNumber_=4;
@@ -234,7 +241,7 @@ for (int i=1;i<=nbfaces_;i++)
   */
 //===========================================================================
 template <class T>
-p1c3d<T>::p1c3d(const char*file) :mesh<tetrahedre,T>(file)
+p1c3d<T>::p1c3d(const char*file) :mesh<tetrahedre<T>,T>(file)
 //===========================================================================
 {
  DofNumber_=4;
@@ -254,7 +261,7 @@ p1c3d<T>::p1c3d(const char*file) :mesh<tetrahedre,T>(file)
   */
 //============================================================================
 template <class T> 
-p1c3d<T>::p1c3d(const p1c3d<T>& EF) :mesh<tetrahedre,T>(EF)
+p1c3d<T>::p1c3d(const p1c3d<T>& EF) :mesh<tetrahedre<T>,T>(EF)
 //============================================================================
 {
  DofNumber_=EF.DofNumber_;
@@ -273,7 +280,7 @@ template <class T>
 p1c3d<T>& p1c3d<T>::operator =(const p1c3d<T>& EF)
 //==========================================================================
 {
- (*this).mesh<tetrahedre,T>::operator=(EF);
+ (*this).mesh<tetrahedre<T>,T>::operator=(EF);
  DofNumber_=EF.DofNumber_;
  GlobalDofNumber_=EF.GlobalDofNumber_;
  RefMassMatrix_=EF.RefMassMatrix_;
@@ -342,12 +349,12 @@ int p1c3d<T>::operator ()(int num_element,int j) const
    \return 1 if EF1 and  EF2 are equal, 0 else
   */
 //==========================================================================
-template <class T>
-int operator ==(const p1c3d<T>& EF1,const p1c3d<T>& EF2)
+template <class FT>
+int operator ==(const p1c3d<FT>& EF1,const p1c3d<FT>& EF2)
 //==========================================================================
 {
  int boolean=1;
- boolean*=(mesh<tetrahedre,T>(EF1)==mesh<tetrahedre,T>(EF2));
+ boolean*=(mesh<tetrahedre<FT>,FT>(EF1)==mesh<tetrahedre<FT>,FT>(EF2));
  boolean*=(EF1.DofNumber_==EF2.DofNumber_);
  boolean*=(EF1.GlobalDofNumber_==EF2.GlobalDofNumber_);
  boolean*=(EF1.RefMassMatrix_==EF2.RefMassMatrix_);
@@ -361,8 +368,8 @@ int operator ==(const p1c3d<T>& EF1,const p1c3d<T>& EF2)
    \return 1 if EF1 and  EF2 are different, 0 else
   */
 //==========================================================================
-template <class T>
-int operator !=(const p1c3d<T>& EF1,const p1c3d<T>& EF2)
+template <class FT>
+int operator !=(const p1c3d<FT>& EF1,const p1c3d<FT>& EF2)
 //==========================================================================
 {
  return(!(EF1==EF2));
@@ -528,7 +535,7 @@ void p1c3d<T>::Dirichlet(matrix<T>& MAT,vector<T>&F,T (*g)(T,T,T)) const
 {
  for(int i=1;i<=GlobalDofNumber_;i++)
   {
-   if(mesh<tetrahedre,T>::operator()(i).boundary_condition())
+   if(mesh<tetrahedre<T>,T>::operator()(i).boundary_condition())
      {
       F[i]=(*g)(vertices[i-1][1],vertices[i-1][2],vertices[i-1][3]);
       for(int j=1;j<=GlobalDofNumber_;j++)
