@@ -2,7 +2,7 @@
  *                 Finite Element Method Object Library        *
  *           class p2c2d : declaration for p2c2d               *
  *                    simula+@metz.ensam.fr                    *
- *	             GNU/linux version 	0.2.0                  *
+ *	             GNU/linux version 	0.2.2                  *
  *            software under General Public License            *
  ***************************************************************
  * copyright  2006 CREUSE Emmanuel
@@ -41,7 +41,7 @@
 	     copyright \htmlonly &#169; \endhtmlonly  2006 SOUALEM Nadir \n
 	     copyright \htmlonly &#169; \endhtmlonly Laboratoire de Physique et Mcanique des Matriaux (LPMM - UMR 7554) \n
 	     copyright \htmlonly &#169; \endhtmlonly Laboratoire de Mathmatiques et ses Applications de Valenciennes (LAMAV - EA 4015)
-    \version 0.2.0
+    \version 0.2.2
     \date 2005-2006
     \bug none
     \warning none
@@ -102,6 +102,31 @@ template <class T> class p2c2d:public mesh_tri_2d<T>
 //=============================
 
 {
+  using mesh<triangle<T>,T>::alive;
+  using mesh<triangle<T>,T>::dimspace_;
+  using mesh<triangle<T>,T>::nbvertex_;
+  using mesh<triangle<T>,T>::nbelement_;
+  using mesh<triangle<T>,T>::vertices;
+  using mesh<triangle<T>,T>::element;
+
+  using mesh_tri_2d<T>::nbsegments_;
+  using mesh_tri_2d<T>::SetOfEdges_;
+  using mesh_tri_2d<T>::Type_of_edge_;
+  using mesh_tri_2d<T>::EdgesPerTriangle_;
+  using mesh_tri_2d<T>::NbTrianglesPerEdge_;
+  using mesh_tri_2d<T>::TrianglesPerEdge_;
+  using mesh_tri_2d<T>::TrianglesPerTriangle_;
+  using mesh_tri_2d<T>::NumberOfNeighboorOfTriangles_;
+  using mesh_tri_2d<T>::NumLocNodeInTriangle_;
+  using mesh_tri_2d<T>:: NormalDirection_;
+  using mesh_tri_2d<T>::hminT_;
+  using mesh_tri_2d<T>::hE_;
+  using mesh_tri_2d<T>::hminE_;
+  using mesh_tri_2d<T>::NormEdge_;
+  using mesh_tri_2d<T>::NormalUnitVector_;
+  using mesh_tri_2d<T>::NbMaxSegmentsPerNode_;
+  using mesh_tri_2d<T>::NbSegmentsPerNode_;
+
   private :
   int DofNumber_;
   int GlobalDofNumber_;
@@ -120,8 +145,8 @@ template <class T> class p2c2d:public mesh_tri_2d<T>
   matrix<T> DiffBasisFunction(T xref=0,T yref=0)const;
   matrix<T> Diff2BasisFunction(T xref=0,T yref=0)const;
   virtual int operator ()(int,int) const;
-  friend  int operator == <T> (const p2c2d<T>&,const p2c2d<T>&);
-  friend  int operator != <T> (const p2c2d<T>&,const p2c2d<T>&);
+  template <class FT> friend  int operator ==  (const p2c2d<FT>&,const p2c2d<FT>&);
+  template <class FT> friend  int operator !=  (const p2c2d<FT>&,const p2c2d<FT>&);
   matrix<T> LocalMassMatrix(int)const;
   matrix<T> LocalStiffnessMatrix(int)const;
   vector<T> Fk(T,T, int) const;
@@ -471,12 +496,12 @@ else
    \return 1 if EF1 and  EF2 are equal, 0 else
   */
 //==========================================================================
-template <class T>
-int operator ==(const p2c2d<T>& EF1,const p2c2d<T>& EF2)
+template <class FT>
+int operator ==(const p2c2d<FT>& EF1,const p2c2d<FT>& EF2)
 //==========================================================================
 {
   int boolean=1;
-  boolean*=(mesh_tri_2d<T>(EF1)==mesh_tri_2d<T>(EF2));
+  boolean*=(mesh_tri_2d<FT>(EF1)==mesh_tri_2d<FT>(EF2));
   boolean*=(EF1.DofNumber_==EF2.DofNumber_);
   boolean*=(EF1.GlobalDofNumber_==EF2.GlobalDofNumber_);
   boolean*=(EF1.RefMassMatrix_==EF2.RefMassMatrix_);
@@ -488,8 +513,8 @@ int operator ==(const p2c2d<T>& EF1,const p2c2d<T>& EF2)
    \return 1 if EF1 and  EF2 are different, 0 else
   */
 //==========================================================================
-template <class T>
-int operator !=(const p2c2d<T>& EF1,const p2c2d<T>& EF2)
+template <class FT>
+int operator !=(const p2c2d<FT>& EF1,const p2c2d<FT>& EF2)
 //==========================================================================
 {
   return(!(EF1==EF2));
@@ -931,4 +956,4 @@ return(sqrt(GlobalError));
 }
 
 
-#endif
\ No newline at end of file
+#endif
