@@ -2,10 +2,10 @@
  *                 Finite Element Method Object Library        *
  *           class q12d : declaration for q12d               *
  *                    simula+@metz.ensam.fr                    *
- *	             GNU/linux version 0.0.8	               *
+ *	             GNU/linux version 0.1.0	               *
  *            software under General Public License            *
  ***************************************************************
- * copyright  2004 CREUSE Emmanuel
+ * copyright  2004,2006 CREUSE Emmanuel
  * copyright  2004 KALETA Olivia
  * copyright  2004 SOUALEM Nadir
  * copyright  Laboratoire de Physique et Mcanique des Matriaux (LPMM - UMR 7554)
@@ -38,13 +38,13 @@
     </FONT>
     \endhtmlonly
 
-    \authors copyright \htmlonly &#169; \endhtmlonly  2004 CREUSE Emmanuel \n
+    \authors copyright \htmlonly &#169; \endhtmlonly  2004,2006 CREUSE Emmanuel \n
 	     copyright \htmlonly &#169; \endhtmlonly  2004 KALETA Olivia \n
 	     copyright \htmlonly &#169; \endhtmlonly  2004 SOUALEM Nadir \n
 	     copyright \htmlonly &#169; \endhtmlonly Laboratoire de Physique et Mcanique des Matriaux (LPMM - UMR 7554) \n
 	     copyright \htmlonly &#169; \endhtmlonly Laboratoire de Mathmatiques et ses Applications de Valenciennes (LAMAV - EA 4015)
-    \version 0.0.8
-    \date 2004
+    \version 0.1.0
+    \date 2004,2006
     \bug none
     \warning none
 */
@@ -84,9 +84,17 @@
 using namespace std;
 
 //-----------------------------------------------------------------------------------------------
-template <class T> class q12d:public mesh <rectangle,T>
+template <class T> class q12d:public mesh <rectangle<T>,T>
 //-----------------------------------------------------------------------------------------------
 {
+
+using mesh<rectangle<T>,T>::alive;
+using mesh<rectangle<T>,T>::dimspace_;
+using mesh<rectangle<T>,T>::nbvertex_;
+using mesh<rectangle<T>,T>::nbelement_;
+using mesh<rectangle<T>,T>::vertices;
+using mesh<rectangle<T>,T>::element;
+
 private :
 int DofNumber_;
 int GlobalDofNumber_;
@@ -115,8 +123,8 @@ matrix<T> RefMassMatrix()const {return (RefMassMatrix_);};
 vector<T> BasisFunction(T,T)const;
 matrix<T> DiffBasisFunction(T xref=0, T yref=0)const;
 virtual int operator ()(int,int)const;
-friend int operator == <T> (const q12d<T>&,const q12d<T>&);
-friend int operator != <T> (const q12d<T>&,const q12d<T>&);
+template <class FT> friend int operator ==  (const q12d<FT>&,const q12d<FT>&);
+template <class FT> friend int operator !=  (const q12d<FT>&,const q12d<FT>&);
 matrix<T> LocalMassMatrix(int)const;
 matrix<T> LocalStiffnessMatrix(int)const;
 vector<T> Fk(T,T,int) const;
@@ -133,7 +141,7 @@ T error_semi_H1(T (*dudx)(T,T),T (*dudy)(T,T),const vector<T>& uh,int NbIntPts)
 */
 //------------------------------------------------------------------------------------------------
 template <class  T>
-q12d <T>::q12d() :mesh <rectangle,T> ()
+q12d <T>::q12d() :mesh <rectangle<T>,T> ()
 //------------------------------------------------------------------------------------------------
 {
 DofNumber_=0;
@@ -150,7 +158,7 @@ GlobalDofNumber_=0;
   */
 //------------------------------------------------------------------------------------------------
 template <class T>
-q12d <T>::q12d(const char*file1,const char*file2) :mesh <rectangle,T> (file1,file2)
+q12d <T>::q12d(const char*file1,const char*file2) :mesh <rectangle<T>,T> (file1,file2)
 //------------------------------------------------------------------------------------------------
 {
 DofNumber_=4;
@@ -171,7 +179,7 @@ RefMassMatrix_(4,1)=1./18.;RefMassMatrix_(4,2)=1./36.;RefMassMatrix_(4,3)=1./18.
 */
 //------------------------------------------------------------------------------------------------
 template <class T>
-q12d<T>::q12d(const char*file) :mesh<rectangle,T>(file)
+q12d<T>::q12d(const char*file) :mesh<rectangle<T>,T>(file)
 //-----------------------------------------------------------------------------------------------
 { 
 DofNumber_=4;
@@ -191,7 +199,7 @@ RefMassMatrix_(4,1)=1./18.;RefMassMatrix_(4,2)=1./36.;RefMassMatrix_(4,3)=1./18.
 
 //------------------------------------------------------------------------------------------------
 template <class T>
-q12d<T>::q12d(const q12d<T>& EF) :mesh<rectangle,T>(EF)
+q12d<T>::q12d(const q12d<T>& EF) :mesh<rectangle<T>,T>(EF)
 //------------------------------------------------------------------------------------------------
 {
 DofNumber_=EF.DofNumber_;
@@ -210,7 +218,7 @@ template <class T>
 q12d<T>& q12d<T>::operator =(const q12d<T>& EF)
 //------------------------------------------------------------------------------------------------
 {
-(*this).mesh<rectangle,T>::operator=(EF);
+(*this).mesh<rectangle<T>,T>::operator=(EF);
 DofNumber_=EF.DofNumber_;
 GlobalDofNumber_=EF.GlobalDofNumber_;
 RefMassMatrix_=EF.RefMassMatrix_;
@@ -276,12 +284,12 @@ return number(element[num_element-1][j]);
    \return 1 if EF1 and  EF2 are equal, 0 else
   */
 //---------------------------------------------------------------------------------------------
-template <class T>
-int operator ==(const q12d<T>& EF1,const q12d<T>& EF2)
+template <class FT>
+int operator ==(const q12d<FT>& EF1,const q12d<FT>& EF2)
 //------------------------------------------------------------------------------------------------
 {
 int boolean=1;
-boolean*=(mesh<rectangle,T>(EF1)==mesh<rectangle,T>(EF2));
+boolean*=(mesh<rectangle<FT>,FT>(EF1)==mesh<rectangle<FT>,FT>(EF2));
 boolean*=(EF1.DofNumber_==EF2.DofNumber_);
 boolean*=(EF1.GlobalDofNumber_==EF2.GlobalDofNumber_);
 boolean*=(EF1.RefMassMatrix_==EF2.RefMassMatrix_);
@@ -295,8 +303,8 @@ return(boolean);
    \return 1 if EF1 and  EF2 are different, 0 else
   */
 //----------------------------------------------------------------------------------------------
-template <class T>
-int operator !=(const q12d<T>& EF1,const q12d<T>& EF2)
+template <class FT>
+int operator !=(const q12d<FT>& EF1,const q12d<FT>& EF2)
 //-----------------------------------------------------------------------------------------------
 {
 return(!(EF1==EF2));
@@ -480,7 +488,7 @@ void q12d<T>::Dirichlet(matrix<T>& MAT,vector<T>&F,T (*g)(T,T)) const
 {
 for(int i=1;i<=GlobalDofNumber_;i++)
 {
-if(mesh<rectangle,T>::operator()(i).boundary_condition())
+if(mesh<rectangle<T>,T>::operator()(i).boundary_condition())
 	{
 F[i]=(*g)(vertices[i-1][1],vertices[i-1][2]);
 for(int j=1;j<=GlobalDofNumber_;j++)
