@@ -2,10 +2,10 @@
  *                 Finite Element Method Object Library        *
  *           class rt02d : declaration for rt02d               *
  *                    simula+@metz.ensam.fr                    *
- *	             GNU/linux version 0.0.8	               *
+ *	             GNU/linux version 0.1.8	               *
  *            software under General Public License            *
  ***************************************************************
- * copyright  2005 CREUSE Emmanuel
+ * copyright  2005-2006 CREUSE Emmanuel
  * copyright  Laboratoire de Physique et Mcanique des Matriaux (LPMM - UMR 7554)
  * copyright  Laboratoire de Mathmatiques et ses Applications de Valenciennes (LAMAV - EA 4015)
  ***************************************************************/
@@ -39,8 +39,8 @@
     \authors copyright \htmlonly &#169; \endhtmlonly  2005 CREUSE Emmanuel \n
 	     copyright \htmlonly &#169; \endhtmlonly Laboratoire de Physique et Mcanique des Matriaux (LPMM - UMR 7554) \n
 	     copyright \htmlonly &#169; \endhtmlonly Laboratoire de Mathmatiques et ses Applications de Valenciennes (LAMAV - EA 4015)
-    \version 0.0.8
-    \date 2005
+    \version 0.1.8
+    \date 2005-2006
     \bug none
     \warning none
 */
@@ -90,6 +90,31 @@ using namespace std;
 template <class T> class rt02d:public mesh_tri_2d<T>
 //=============================
 {
+  using mesh<triangle<T>,T>::alive;
+  using mesh<triangle<T>,T>::dimspace_;
+  using mesh<triangle<T>,T>::nbvertex_;
+  using mesh<triangle<T>,T>::nbelement_;
+  using mesh<triangle<T>,T>::vertices;
+  using mesh<triangle<T>,T>::element;
+
+  using mesh_tri_2d<T>::nbsegments_;
+  using mesh_tri_2d<T>::SetOfEdges_;
+  using mesh_tri_2d<T>::Type_of_edge_;
+  using mesh_tri_2d<T>::EdgesPerTriangle_;
+  using mesh_tri_2d<T>::NbTrianglesPerEdge_;
+  using mesh_tri_2d<T>::TrianglesPerEdge_;
+  using mesh_tri_2d<T>::TrianglesPerTriangle_;
+  using mesh_tri_2d<T>::NumberOfNeighboorOfTriangles_;
+  using mesh_tri_2d<T>::NumLocNodeInTriangle_;
+  using mesh_tri_2d<T>:: NormalDirection_;
+  using mesh_tri_2d<T>::hminT_;
+  using mesh_tri_2d<T>::hE_;
+  using mesh_tri_2d<T>::hminE_;
+  using mesh_tri_2d<T>::NormEdge_;
+  using mesh_tri_2d<T>::NormalUnitVector_;
+  using mesh_tri_2d<T>::NbMaxSegmentsPerNode_;
+  using mesh_tri_2d<T>::NbSegmentsPerNode_;
+
   private :
   int DofNumber_;
   int GlobalDofNumber_;
@@ -99,8 +124,8 @@ template <class T> class rt02d:public mesh_tri_2d<T>
     rt02d(const char*);
     rt02d(const rt02d<T>&);
     rt02d<T>& operator=(const rt02d<T>&);
-    friend  int operator == <T> (const rt02d<T>&,const rt02d<T>&);
-    friend  int operator != <T> (const rt02d<T>&,const rt02d<T>&);
+    template<class FT> friend  int operator ==  (const rt02d<FT>&,const rt02d<FT>&);
+    template<class FT> friend  int operator !=  (const rt02d<FT>&,const rt02d<FT>&);
     int operator()(int,int) const;
     int DofNumber()const{return (DofNumber_);};
     int GlobalDofNumber()const{return (GlobalDofNumber_);};
@@ -116,6 +141,7 @@ template <class T> class rt02d:public mesh_tri_2d<T>
     void Compute_p(vector<T>&, const vector<T>&, const vector<T>&,const vector<T>&, T,int);
     T error_p(const vector<T>&,T (*px)(T,T),T (*py)(T,T),T,int) ;
     T error_u(const vector<T>&,T,T (*u)(T,T),int) ;
+    T error_u_loc2(const vector<T>&,T,T (*u)(T,T),int,int) ;
     T error_loc_u(T,T (*u)(T,T),int,int) ;
     T eta_11_K_p(const vector<T>&,T,int,int);
     T eta_13_K_p(const vector<T>&,T,int);
@@ -124,6 +150,12 @@ template <class T> class rt02d:public mesh_tri_2d<T>
     T eta_2(T (*f)(T,T),T,int);
     void refine_FE(vector<int>&); // refine the Finite Element
     void refine_FE_angle(vector<int>&, T angle); // refine the Finite Element
+    matrix<T> LocalMassMatrix(int)const;
+    spmatrix<T> UV() const;
+    T error_div (T (*divp)(T,T),const vector<T>& uh,int NbIntPts) const ;
+    T error_L2(const vector<T>&,T (*px)(T,T),T (*py)(T,T),int) ;
+    T error_L2_u(const vector<T>&,T (*u)(T,T),int); 
+    T error_L2_loc2(const vector<T>&,T (*px)(T,T),T (*py)(T,T),int,int);
 };
 /*!
   \brief default constructor of the rt02d class
@@ -235,12 +267,12 @@ return EdgesPerTriangle_(num_element,j);
    \return 1 if EF1 and  EF2 are equal, 0 else
   */
 //==========================================================================
-template <class T>
-int operator ==(const rt02d<T>& EF1,const rt02d<T>& EF2)
+template <class FT>
+int operator ==(const rt02d<FT>& EF1,const rt02d<FT>& EF2)
 //==========================================================================
 {
   int boolean=1;
-  boolean*=(mesh_tri_2d<T>(EF1)==mesh_tri_2d<T>(EF2));
+  boolean*=(mesh_tri_2d<FT>(EF1)==mesh_tri_2d<FT>(EF2));
   boolean*=(EF1.DofNumber_==EF2.DofNumber_);
   boolean*=(EF1.GlobalDofNumber_==EF2.GlobalDofNumber_);
   return(boolean);
@@ -252,8 +284,8 @@ int operator ==(const rt02d<T>& EF1,const rt02d<T>& EF2)
    \return 1 if EF1 and  EF2 are different, 0 else
   */
 //==========================================================================
-template <class T>
-int operator !=(const rt02d<T>& EF1,const rt02d<T>& EF2)
+template <class FT>
+int operator !=(const rt02d<FT>& EF1,const rt02d<FT>& EF2)
 //==========================================================================
 {
   return(!(EF1==EF2));
@@ -462,7 +494,7 @@ matrix<T> rt02d<T>::Ck(int num_element) const
 assert((num_element>=1)&&(num_element<=nbelement_));
 matrix<T> M(3,3);
 for (int i=1; i<=3; i++)
- M(i,i)=NormalDirection(num_element,i);
+ M(i,i)=NormalDirection_(num_element,i);
 return(M);
 }
 
@@ -594,8 +626,8 @@ for (int num_element=1; num_element<=nbelement_; num_element++)
 	  vec[1]=0; vec[2]=0;
 	   for (int j=1; j<=3; j++)
 	    {
-	    vec[1]+=P[(*this)(num_element,j)]*BasisFunction(1,j)*(T)NormalDirection_(num_element,j);
-	    vec[2]+=P[(*this)(num_element,j)]*BasisFunction(2,j)*(T)NormalDirection_(num_element,j);
+	    vec[1]+=P[(*this)(num_element,j)]*BasisFunction(1,j)*NormalDirection_(num_element,j);
+	    vec[2]+=P[(*this)(num_element,j)]*BasisFunction(2,j)*NormalDirection_(num_element,j);
 	    }
 	  vec/=element[num_element-1].AbsDet();
 //	  if (NormGradu(Fxkl[1],Fxkl[2])<epsilon)
@@ -668,7 +700,35 @@ for (int num_element=1; num_element<=nbelement_; num_element++)
 }
   return (powf(error,1.0/s));
 }
+//====================================================
+template <class T> T rt02d<T>::error_u_loc2(const vector<T>& US, T s,T (*u)(T,T),int NbIntPts,int num_element) 
+//====================================================
+{
 
+T omegak, omegal, xk, xl;
+vector<T> Fxkl(2);
+integration<T> itg;
+itg.Gauss(NbIntPts,0.0,1.0);
+
+T error_loc;
+
+ error_loc=0;
+  for (int k=1; k<=itg.NbIntPts(); k++)
+    {
+      omegak=itg.weight(k);
+      xk=itg.point(k);
+
+      for (int l=1; l<=itg.NbIntPts(); l++)
+	{
+	  omegal=itg.weight(l);
+          xl=(1-xk)*itg.point(l);
+	  Fxkl=Fk(xk,xl,num_element);
+	  error_loc+=omegak*omegal*(1-xk)*powf(abs(u(Fxkl[1],Fxkl[2])-US[num_element]),s);
+	 }
+    }
+  error_loc*=element[num_element-1].AbsDet();
+  return (error_loc);
+}
 /*!
    \param Umaille type vector<T>& (contains the value of \f$u\f$ on each triangle of the mesh)
    \param u type (*f)(T,T)
@@ -714,7 +774,7 @@ template <class T>
 void rt02d<T>::refine_FE(vector<int> & TABTRI)
 //=============================
 {
-  refine_mesh(TABTRI);
+  (*this).refine_mesh(TABTRI);
   GlobalDofNumber_=nbsegments_;
 }
 
@@ -727,7 +787,7 @@ template <class T>
 void rt02d<T>::refine_FE_angle(vector<int> & TABTRI, T angle)
 //=============================
 {
-  refine_mesh_angle(TABTRI,angle);
+  (*this).refine_mesh_angle(TABTRI,angle);
   GlobalDofNumber_=nbsegments_;
 }
 
@@ -867,8 +927,241 @@ value+=eta_2_K_q(f,s,NbPts,num_element);
 return (powf(value,1.0/s)); 
 }
 
+/*!
+  \param num_element element's number
+  \return local mass matrix of the element num_element
+  */
+//===========================================================
+template <class T> 
+matrix<T> rt02d<T>::LocalMassMatrix(int num_element) const
+//===========================================================
+{
 
-	 
+// ici on a choisi 5 points d'intgration ce qui est trop important
+// compte tenu du degr des fonctions  integrer... A faire baisser 
+// la valeur tout juste suffisante. 
+
+int NbIntPts=5;
+integration<T> itg;
+itg.Gauss(NbIntPts,0.0,1.0);
+
+
+assert((num_element>=1)&&(num_element<=nbelement_));
+matrix<T> matloc(3,3);
+matrix<T> B=t(element[num_element-1].Mk())*element[num_element-1].Mk()/element[num_element-1].AbsDet();
+T valeurint;
+T xk,yj,omegak,omegaj;
+vector<T> phim(2),phip(2);
+matrix<T> valeurfonctions(2,3);
+
+for (int m=1; m<=3; m++)
+  {
+    for (int p=1; p<=3; p++)
+      {
+        valeurint=0.0;
+        for (int k=1; k<=NbIntPts; k++) 
+         {
+          omegak=itg.weight(k);
+          xk=itg.point(k); 
+          for (int j=1; j<=NbIntPts; j++)
+	   {
+	    omegaj=itg.weight(j);
+            yj=itg.point(j); 
+              valeurfonctions=RefBasisFunction(xk,(1-xk)*yj);
+              for (int l=1; l<=2; l++)
+		{ 
+		  phim[l]=valeurfonctions(l,m);
+		  phip[l]=valeurfonctions(l,p);
+                }
+              valeurint+=omegaj*omegak*abs(1.0-xk)*((B*phim)*phip);	     
+           }
+	 }
+	matloc(m,p)=NormalDirection_(num_element,m)*NormalDirection_(num_element,p)*valeurint;
+      }
+  }
+ return matloc;
+}
+/*!
+  \return the global matrix of the tetraRT0 element
+*/
+//====================================================
+template <class T>
+spmatrix<T> rt02d<T>::UV()const
+//====================================================
+{
+matrix<T> Mass(DofNumber_,DofNumber_);
+spmatrix<T> UV_(GlobalDofNumber_,GlobalDofNumber_,7);
+for(int numele=1;numele<=nbelement_;numele++)
+	{
+	        Mass=LocalMassMatrix(numele);
+		for(int i=1;i<=DofNumber_;i++)
+		for(int j=1;j<=DofNumber_;j++)
+		UV_.add(EdgesPerTriangle_(numele,i),EdgesPerTriangle_(numele,j),Mass(i,j));
+	}
+
+return UV_;
+}
+//======================================================
+template <class T>
+T rt02d<T>::error_div (T (*divp)(T,T),const vector<T>& uh,int NbIntPts) const
+//======================================================
+{
+  T GlobalError=0;
+  T LocalError2;
+  T Somme;
+  T  Determinant;
+
+  integration<T> itg;
+  itg.Gauss(NbIntPts,0.0,1.0);
+
+  T xk,yj,omegak,omegaj;
+ 
+  GlobalError=0.0;
+
+  for(int num_element=1;num_element<=nbelement_;num_element++)
+  {
+
+   Determinant=element[num_element-1].AbsDet() ;
+
+   LocalError2=0; 
+   for (int k=1; k<=itg.NbIntPts(); k++) 
+   {
+    omegak=itg.weight(k);
+    xk=itg.point(k); 
+    for (int j=1; j<=itg.NbIntPts(); j++)
+     {
+      omegaj=itg.weight(j);
+      yj=itg.point(j); 
+  
+        Somme=0;
+
+	   for (int l=1; l<=3; l++)
+             Somme+=uh[EdgesPerTriangle_(num_element,l)]*NormalDirection_(num_element,l);
+          
+             LocalError2+=omegaj*omegak*abs(1.0-xk)*
+	     pow((divp(Fk(xk,(1-xk)*yj,num_element)[1],
+                Fk(xk,(1-xk)*yj,num_element)[2])
+	       -2.0*Somme/Determinant),2);
+     }
+   }
+   GlobalError+=Determinant*LocalError2;
+  }
+  
+return(sqrt(GlobalError));
+}
+//====================================================
+template <class T> T rt02d<T>::error_L2(const vector<T>& P, T (*px)(T,T), T (*py)(T,T), int NbIntPts) 
+//====================================================
+{
+T omegak, omegal, xk, xl;
+T valeur_px, valeur_py;
+vector<T> Fxkl(2), vec(2);
+integration<T> itg;
+itg.Gauss(NbIntPts,0.0,1.0);
+matrix<T> Mk(2,2),BasisFunction(2,3);
+T error=0;
+T error_loc;
+for (int num_element=1; num_element<=nbelement_; num_element++)
+{
+ error_loc=0;
+ Mk=element[num_element-1].Mk();
+  for (int k=1; k<=itg.NbIntPts(); k++)
+    {
+      omegak=itg.weight(k);
+      xk=itg.point(k);
+      for (int l=1; l<=itg.NbIntPts(); l++)
+	{
+	  omegal=itg.weight(l);
+          xl=(1-xk)*itg.point(l);
+	  Fxkl=Fk(xk,xl,num_element);
+	  BasisFunction=Mk*RefBasisFunction(xk,xl);
+	  vec[1]=0; vec[2]=0;
+	   for (int j=1; j<=3; j++)
+	    {
+ 	    vec[1]+=P[(*this)(num_element,j)]*BasisFunction(1,j)*NormalDirection_(num_element,j);
+ 	    vec[2]+=P[(*this)(num_element,j)]*BasisFunction(2,j)*NormalDirection_(num_element,j);
+	    }
+	  vec=vec/(element[num_element-1].AbsDet());
+	    valeur_px=px(Fxkl[1],Fxkl[2]);
+	    valeur_py=py(Fxkl[1],Fxkl[2]);
+	  error_loc+=omegak*omegal*(1-xk)*(pow((valeur_px-vec[1]),2)+pow((valeur_py-vec[2]),2));	  
+	 }
+    }
+  error_loc*=element[num_element-1].AbsDet();
+  error+=error_loc;  
+}
+return (sqrt(error));
+}
+
+//====================================================
+template <class T> T rt02d<T>::error_L2_loc2(const vector<T>& P, T (*px)(T,T), T (*py)(T,T), int NbIntPts, int num_element) 
+//====================================================
+{
+T omegak, omegal, xk, xl;
+T valeur_px, valeur_py;
+vector<T> Fxkl(2), vec(2);
+integration<T> itg;
+itg.Gauss(NbIntPts,0.0,1.0);
+matrix<T> Mk(2,2),BasisFunction(2,3);
+T error_loc2=0;
+
+ Mk=element[num_element-1].Mk();
+  for (int k=1; k<=itg.NbIntPts(); k++)
+    {
+      omegak=itg.weight(k);
+      xk=itg.point(k);
+      for (int l=1; l<=itg.NbIntPts(); l++)
+	{
+	  omegal=itg.weight(l);
+          xl=(1-xk)*itg.point(l);
+	  Fxkl=Fk(xk,xl,num_element);
+	  BasisFunction=Mk*RefBasisFunction(xk,xl);
+	  vec[1]=0; vec[2]=0;
+	   for (int j=1; j<=3; j++)
+	    {
+ 	    vec[1]+=P[(*this)(num_element,j)]*BasisFunction(1,j)*NormalDirection_(num_element,j);
+ 	    vec[2]+=P[(*this)(num_element,j)]*BasisFunction(2,j)*NormalDirection_(num_element,j);
+	    }
+	  vec=vec/(element[num_element-1].AbsDet());
+	    valeur_px=px(Fxkl[1],Fxkl[2]);
+	    valeur_py=py(Fxkl[1],Fxkl[2]);
+	  error_loc2+=omegak*omegal*(1-xk)*(pow((valeur_px-vec[1]),2)+pow((valeur_py-vec[2]),2));	  
+	 }
+    }
+  error_loc2*=element[num_element-1].AbsDet();
+  return (error_loc2);  
+}
+//====================================================
+template <class T> T rt02d<T>::error_L2_u(const vector<T>& uh, T (*u)(T,T), int NbIntPts) 
+//====================================================
+{
+T omegak, omegal, xk, xl;
+T valeur_u;
+integration<T> itg;
+itg.Gauss(NbIntPts,0.0,1.0);
+T error=0;
+T error_loc;
+
+for (int num_element=1; num_element<=nbelement_; num_element++)
+{
+ error_loc=0;
+  for (int k=1; k<=itg.NbIntPts(); k++)
+    {
+      omegak=itg.weight(k);
+      xk=itg.point(k);
+      for (int l=1; l<=itg.NbIntPts(); l++)
+	{
+	  omegal=itg.weight(l);
+          xl=(1-xk)*itg.point(l);
+	  valeur_u=u(Fk(xk,xl,num_element)[1],Fk(xk,xl,num_element)[2]);
+	  error_loc+=omegak*omegal*(1-xk)*(pow((valeur_u-uh[num_element]),2));	  
+	 }
+    }
+  error_loc*=element[num_element-1].AbsDet();
+  error+=error_loc;  
+}
+return (sqrt(error));
+}	 
 #endif
 
 
