@@ -2,10 +2,10 @@
  *                 Finite Element Method Object Library        *
  *           class tetraRT0 : declaration for tetraRT0         *
  *                    simula+@metz.ensam.fr                    *
- *	             GNU/linux version 0.2.6	               *
+ *	             GNU/linux version 0.2.8	               *
  *            software under General Public License            *
  ***************************************************************
- * copyright  2003,2004 CREUSE Emmanuel
+ * copyright  2003,2004,2006 CREUSE Emmanuel
  * copyright  2004 PIRAUT Frdric
  * copyright  Laboratoire de Physique et Mcanique des Matriaux (LPMM - UMR 7554)
  * copyright  Laboratoire de Mathmatiques et ses Applications de Valenciennes (LAMAV - EA 4015)
@@ -37,12 +37,12 @@
     </FONT>
     \endhtmlonly
 
-    \authors copyright \htmlonly &#169; \endhtmlonly  2003,2004 CREUSE Emmanuel \n
+    \authors copyright \htmlonly &#169; \endhtmlonly  2003,2004,2006 CREUSE Emmanuel \n
 	     copyright \htmlonly &#169; \endhtmlonly  2004 PIRAUT Frdric \n
 	     copyright \htmlonly &#169; \endhtmlonly Laboratoire de Physique et Mcanique des Matriaux (LPMM - UMR 7554) \n
 	     copyright \htmlonly &#169; \endhtmlonly Laboratoire de Mathmatiques et ses Applications de Valenciennes (LAMAV - EA 4015)
-    \version 0.2.6
-    \date 2003-2004
+    \version 0.2.8
+    \date 2003-2006
     \bug none
     \warning none
 */
@@ -83,9 +83,17 @@ using namespace std;
 
 
 //=============================
-template <class T> class tetraRT0:public mesh<tetrahedre,T>
+template <class T> class tetraRT0:public mesh<tetrahedre<T>,T>
 //=============================
 {
+ 
+  using mesh<tetrahedre<T>,T>::alive;
+  using mesh<tetrahedre<T>,T>::dimspace_;
+  using mesh<tetrahedre<T>,T>::nbvertex_;
+  using mesh<tetrahedre<T>,T>::nbelement_;
+  using mesh<tetrahedre<T>,T>::vertices;
+  using mesh<tetrahedre<T>,T>::element;
+
   private :
 int DofNumber_;
 int GlobalDofNumber_;
@@ -166,8 +174,8 @@ matrix<T> normal_;
   matrix<T> LocalMassMatrix(int)const;
   //  int boundary(vector<int>& numero_des_noeuds,const char*geometrie)const;
   virtual int operator ()(int,int) const;
-  friend  int operator == <T> (const tetraRT0<T>&,const tetraRT0<T>&);
-  friend  int operator != <T> (const tetraRT0<T>&,const tetraRT0<T>&);
+  template <class FT> friend  int operator ==  (const tetraRT0<FT>&,const tetraRT0<FT>&);
+  template <class FT> friend  int operator !=  (const tetraRT0<FT>&,const tetraRT0<FT>&);
   vector<T> Fk(T,T,T, int) const;
   vector<T> Fkm1(T,T,T,int) const;
   vector<T> vect(const vector<T>&, const vector<T>&) const; // vectorial product (this function should be developped
@@ -187,7 +195,7 @@ matrix<T> normal_;
            
 //=============================
 template <class T> 
-tetraRT0<T>::tetraRT0() :mesh<tetrahedre,T>()
+tetraRT0<T>::tetraRT0() :mesh<tetrahedre<T>,T>()
 //=============================
 {
 DofNumber_=0;
@@ -207,7 +215,7 @@ nbfaces_=0;
 
 //===============================
 template <class T>
-tetraRT0<T>::tetraRT0(const char*file1,const char*file2):mesh<tetrahedre,T>(file1,file2)
+tetraRT0<T>::tetraRT0(const char*file1,const char*file2):mesh<tetrahedre<T>,T>(file1,file2)
 //===============================
 
 {
@@ -351,7 +359,7 @@ for (int num_element=1; num_element<=nbelement_; num_element++)
   */
 //=============================== 
  template <class T> 
-tetraRT0<T>::tetraRT0(const char*file1):mesh<tetrahedre,T>(file1)
+tetraRT0<T>::tetraRT0(const char*file1):mesh<tetrahedre<T>,T>(file1)
 //===============================
 {
  DofNumber_=4;
@@ -490,7 +498,7 @@ for (int num_element=1; num_element<=nbelement_; num_element++)
   */
 //=============================
 template <class T> 
-tetraRT0<T>::tetraRT0(const tetraRT0<T>& EF) :mesh<tetrahedre,T>(EF)
+tetraRT0<T>::tetraRT0(const tetraRT0<T>& EF) :mesh<tetrahedre<T>,T>(EF)
 //=============================
 {
   DofNumber_=EF.DofNumber_;
@@ -514,7 +522,7 @@ template <class T>
 tetraRT0<T>& tetraRT0<T>::operator =(const tetraRT0<T>& EF)
 //========================================================
 {
-      (*this).mesh<tetrahedre,T>::operator=(EF);
+      (*this).mesh<tetrahedre<T>,T>::operator=(EF);
       DofNumber_=EF.DofNumber_;
       GlobalDofNumber_=EF.GlobalDofNumber_;
       nbfaces_=EF.nbfaces_;
@@ -631,12 +639,12 @@ return face_of_element_(num_element,j);
    \return 1 if EF1 and  EF2 are equal, 0 else
   */
 //==========================================================================
-template <class T>
-int operator ==(const tetraRT0<T>& EF1,const tetraRT0<T>& EF2)
+template <class FT>
+int operator ==(const tetraRT0<FT>& EF1,const tetraRT0<FT>& EF2)
 //==========================================================================
 {
   int boolean=1;
-  boolean*=(mesh<tetrahedre,T>(EF1)==mesh<tetrahedre,T>(EF2));
+  boolean*=(mesh<tetrahedre<FT>,FT>(EF1)==mesh<tetrahedre<FT>,FT>(EF2));
   boolean*=(EF1.DofNumber_==EF2.DofNumber_);
   boolean*=(EF1.GlobalDofNumber_==EF2.GlobalDofNumber_);
   boolean*=(EF1.nbfaces_==EF2.nbfaces_);
@@ -656,8 +664,8 @@ int operator ==(const tetraRT0<T>& EF1,const tetraRT0<T>& EF2)
    \return 1 if EF1 and  EF2 are different, 0 else
   */
 //==========================================================================
-template <class T>
-int operator !=(const tetraRT0<T>& EF1,const tetraRT0<T>& EF2)
+template <class FT>
+int operator !=(const tetraRT0<FT>& EF1,const tetraRT0<FT>& EF2)
 //==========================================================================
 {
   return(!(EF1==EF2));
