@@ -2,7 +2,7 @@
  *                 Finite Element Method Object Library        *
  *           class mesh : declaration for mesh                 *
  *                    simula+@metz.ensam.fr                    *
- *	             GNU/linux version 0.2.4	               *
+ *	             GNU/linux version 0.3.0	               *
  *            software under General Public License            *
  ***************************************************************
  * copyright  2002,2003,2004,2005,2006 CREUSE Emmanuel
@@ -41,7 +41,7 @@
 	     copyright \htmlonly &#169; \endhtmlonly  2002,2003,2004,2005,2006 SOUALEM Nadir \n
 	     copyright \htmlonly &#169; \endhtmlonly Laboratoire de Physique et Mcanique des Matriaux (LPMM - UMR 7554) \n
 	     copyright \htmlonly &#169; \endhtmlonly Laboratoire de Mathmatiques et ses Applications de Valenciennes (LAMAV - EA 4015)
-    \version 0.2.4
+    \version 0.3.0
     \date 2002-2006
     \bug none
     \warning none
@@ -95,12 +95,8 @@ elements<====>|   O____O______\O______O <===========|
 #define _mesh_h
 
 using namespace std;
-/*we define a "template of template":
-template <class obj,class T>
-where obj is a template specification
-*/
 
-template <template <typename U> class obj,class T>
+template <class obj,class T>
 class mesh
 {
 protected:
@@ -109,10 +105,10 @@ protected:
   int nbvertex_;//.........................number of vertices
   int nbelement_;//........................number of elements
   node<T>* vertices;//.....................array of vertices
-  obj<T>* element;//.......................array of elements
+  obj* element;//.......................array of elements
 public:
   mesh(){alive=0;};//......................default constructor
-  mesh(const char*,const char*);//constructor (file of nodes,file of elements)
+      mesh(const char*,const char*);//constructor (file of nodes,file of elements)
   mesh(const char*);//............constructor (file of nodes and element)
   mesh(const mesh<obj,T>&);//.....copy constructor
   mesh<obj,T>& operator=(const mesh<obj,T>&);//mesh affectation
@@ -120,15 +116,15 @@ public:
   int dimspace() {return dimspace_;};//........returns space dimension
   int nbvertex() {return nbvertex_;};//........returns number of vertex
   int nbelement(){return nbelement_;};//.......returns number of element
-  obj<T>& operator[](int)const;   //...........returns or modify an element
+  obj& operator[](int)const;   //...........returns or modify an element
   node<T>& operator () (int)const;//...........returns or modify a node
   void make_anisotropic( const T&, const int&) const; // make the mesh anisotropic
   int number (const node<T>&) const;//...............returns the number of the node
-  friend ostream& operator << <obj,T> (ostream&, const mesh<obj,T>&);
-  //...........................................cout<<mesh
-  friend  int operator == <obj,T> (const mesh<obj,T>&, const mesh<obj,T>&);
-  //...........................................1 if the mesh are equal else O
-  void visu2d() const;//.........................mesh visualisation
+  template <class Fobj, class FT> friend ostream& operator << (ostream&, const mesh<Fobj,FT>&);
+  template <class Fobj, class FT> friend  int operator == (const mesh<Fobj,FT>&, const mesh<Fobj,FT>&);
+//   //...........................................1 if the mesh are equal else O
+   void visu2d() const;//.........................mesh visualisation
+  node<T>& vertex(int num) {return vertices[num-1];};
 };
 
 
@@ -140,8 +136,7 @@ public:
   \param filename2 file with elements
   */
 //=============================================================================
-template <template <typename U> class obj,class T>
-mesh<obj,T>::mesh(const char* filename1,const char* filename2)
+template <class obj, class T> mesh<obj,T>::mesh(const char* filename1,const char* filename2)
 //=============================================================================
 {
 //=====================
@@ -152,7 +147,9 @@ mesh<obj,T>::mesh(const char* filename1,const char* filename2)
   assert (!file1.fail());
   file1 >>dimspace_;
   file1 >> nbvertex_;
-  vertices=new node<T>[nbvertex_](dimspace_);
+  vertices = new node<T>[nbvertex_];
+  for (int i=0; i<nbvertex_; i++)
+       vertices[i] = node<T>(dimspace_);
   for(int i=0;i<nbvertex_;++i)
     file1>>vertices[i];
   file1.close();
@@ -162,8 +159,7 @@ mesh<obj,T>::mesh(const char* filename1,const char* filename2)
   ifstream file2(filename2);
   assert (!file2.fail());
   file2 >> nbelement_;
-  element=new obj<T>[nbelement_];
-  
+  element=new obj[nbelement_];
   node<T> ** nodes_of_element;
   nodes_of_element=new node<T>*[element[0].nb_nodes()];
   int num;
@@ -175,7 +171,7 @@ mesh<obj,T>::mesh(const char* filename1,const char* filename2)
          file2 >> num;
          nodes_of_element[j]=&vertices[num-1];
         }
-      element[i]=obj<T>(nodes_of_element);
+        element[i]=obj(nodes_of_element); 
     }
   delete [] nodes_of_element;
   file2.close();
@@ -188,8 +184,7 @@ mesh<obj,T>::mesh(const char* filename1,const char* filename2)
   \param filename file with nodes and elements
   */
 //======================================================
-template <template <typename U> class obj,class T>
-mesh<obj,T>::mesh(const char* filename)
+template <class obj, class T> mesh<obj,T>::mesh(const char* filename)
 //======================================================
 {
 //=====================
@@ -200,11 +195,13 @@ mesh<obj,T>::mesh(const char* filename)
   assert (!file.fail());
   file >>dimspace_;
   file >> nbvertex_;
-  vertices=new node<T>[nbvertex_](dimspace_);
+  vertices = new node<T>[nbvertex_];
+  for (int i=0; i<nbvertex_; i++)
+       vertices[i] = node<T>(dimspace_);
   for(int i=0;i<nbvertex_;++i)
       file >> vertices[i];
   file>>nbelement_;
-  element=new obj<T>[nbelement_];
+  element=new obj[nbelement_];
 
   node<T> ** nodes_of_element;
   nodes_of_element=new node<T>*[element[0].nb_nodes()];
@@ -217,7 +214,7 @@ mesh<obj,T>::mesh(const char* filename)
          file >> num;
          nodes_of_element[j]=&vertices[num-1];
         }
-      element[i]=obj<T>(nodes_of_element);
+      element[i]=obj(nodes_of_element);
     }
   delete [] nodes_of_element;
   file.close();
@@ -229,20 +226,20 @@ mesh<obj,T>::mesh(const char* filename)
   \param Th mesh to duplicate
 */
 //======================================================
-template <template <typename U> class obj,class T>
-mesh<obj,T>::mesh(const mesh<obj,T> & Th)
+template <class obj, class T> mesh<obj,T>::mesh(const mesh<obj,T> & Th)
 //======================================================
 {
   alive=Th.alive;
   dimspace_=Th.dimspace_;
   nbvertex_=Th.nbvertex_;
   nbelement_=Th.nbelement_;
-  
-  vertices=new node<T>[nbvertex_](dimspace_);
+   vertices = new node<T>[nbvertex_];
+  for (int i=0; i<nbvertex_; i++)
+       vertices[i] = node<T>(dimspace_);
   for(int i=0;i<nbvertex_;i++)
       vertices[i]=Th.vertices[i];
   
-  element=new obj<T>[nbelement_];
+  element=new obj[nbelement_];
   
   node<T> ** nodes_of_element;
   nodes_of_element=new node<T>*[element[0].nb_nodes()]; 
@@ -254,7 +251,7 @@ for(int i=0;i<nbelement_;i++)
         nodes_of_element[j]=&vertices[Th.number(Th[i+1][j+1])-1];
       }
    
-    element[i]=obj<T>(nodes_of_element);
+    element[i]=obj(nodes_of_element);
   }
  
 delete[] nodes_of_element;
@@ -268,8 +265,7 @@ delete[] nodes_of_element;
   \n\n
 */
 //========================================================
-template <template <typename U> class obj,class T>
-mesh<obj,T>& mesh<obj,T>::operator =(const mesh<obj,T> & Th)
+template <class obj, class T> mesh<obj,T>& mesh<obj,T>::operator =(const mesh<obj,T> & Th)
 //========================================================
 {
   assert(Th.alive);
@@ -277,8 +273,11 @@ mesh<obj,T>& mesh<obj,T>::operator =(const mesh<obj,T> & Th)
   {
   alive=1;
   dimspace_=Th.dimspace_;
-  vertices=new node<T>[nbvertex_=Th.nbvertex_](dimspace_);
-  element=new obj<T>[nbelement_=Th.nbelement_];
+  nbvertex_=Th.nbvertex_;
+  vertices = new node<T>[nbvertex_];
+  for (int i=0; i<nbvertex_; i++)
+       vertices[i] = node<T>(dimspace_);
+  element=new obj[nbelement_=Th.nbelement_];
   }
   else
   {
@@ -300,7 +299,7 @@ mesh<obj,T>& mesh<obj,T>::operator =(const mesh<obj,T> & Th)
         nodes_of_element[j]=&vertices[Th.number(Th[i+1][j+1])-1];
       }
    
-    element[i]=obj<T>(nodes_of_element);  
+    element[i]=obj(nodes_of_element);  
 
     }
   delete[] nodes_of_element;
@@ -313,8 +312,7 @@ return(*this);
   Deletes vertices and element and set the scalar to 0 
 */
 //===================================
-template <template <typename U> class obj,class T>
-mesh<obj,T>::~mesh()
+template <class obj, class T> mesh<obj,T>::~mesh()
 //===================================
 {
 
@@ -337,8 +335,7 @@ mesh<obj,T>::~mesh()
 */
 
 //==========================================================================
-template <template <typename U> class obj,class T>
-obj<T>& mesh<obj,T>:: operator[] (int k)const
+template <class obj, class T> obj& mesh<obj,T>:: operator[] (int k)const
 //==========================================================================
 {
   assert((k>0)&&(k<=nbelement_));
@@ -352,8 +349,7 @@ obj<T>& mesh<obj,T>:: operator[] (int k)const
   \return the node k of the mesh \n \n
 */
 //==========================================================================
-template <template <typename U> class obj,class T>
-node<T>& mesh<obj,T>:: operator() (int k)const
+template <class obj, class T> node<T>& mesh<obj,T>:: operator() (int k)const
 //==========================================================================
 {
   assert((k>0)&&(k<=nbvertex_));
@@ -365,8 +361,10 @@ node<T>& mesh<obj,T>:: operator() (int k)const
   \param Th triangle
 */
 //==========================================================================
-template <template <typename U> class obj,class T>
-ostream& operator << (ostream& exit, const mesh<obj,T>& Th)
+//template <template <typename U> class obj,class T>
+//ostream& operator << (ostream& exit, const mesh<obj,T>& Th)
+//template <template <class> class Fobj, class FT> ostream& operator << (ostream& exit, const mesh<Fobj,FT>& Th)
+template <class Fobj, class FT> ostream& operator << (ostream& exit, const mesh<Fobj,FT>& Th)
 //==========================================================================
 {
   assert(Th.alive);
@@ -378,14 +376,8 @@ ostream& operator << (ostream& exit, const mesh<obj,T>& Th)
   exit<<"_______________________"<<"\n";
   exit<<"Nodes        |"<<setw(8)<<Th.nbvertex_<<"|\n";
   exit<<"_______________________"<<"\n";
-/*  exit<<"Data1        |"<<setw(8)<<1111111<<"|\n";
-  exit<<"_______________________"<<"\n";
-  exit<<"Data2        |"<<setw(8)<<2222222<<"|\n";
-  exit<<"_______________________"<<"\n";
-  exit<<"Data3        |"<<setw(8)<<3333333<<"|\n";
-  exit<<"_______________________"<<"\n";*/
   exit<<endl;
-  return(exit);
+   return(exit);
 }
 
 /*!
@@ -394,8 +386,7 @@ ostream& operator << (ostream& exit, const mesh<obj,T>& Th)
   \return int The global number of node A in the mesh
 */
 //==========================================================================
-template <template <typename U> class obj,class T>
-int mesh<obj,T>::number(const node<T> &A) const
+template <class obj, class T> int mesh<obj,T>::number(const node<T> &A) const
 //==========================================================================
 {
   
@@ -410,8 +401,7 @@ int mesh<obj,T>::number(const node<T> &A) const
   \param cas allows to give the way to refine (read the function itself)
 */
 //==========================================================================
-template <template <typename U> class obj,class T>
-void mesh<obj,T>::make_anisotropic(const T & para_tau, const int & cas) const
+template <class obj, class T> void mesh<obj,T>::make_anisotropic(const T & para_tau, const int & cas) const
 //==========================================================================
 {
 
@@ -517,9 +507,11 @@ void mesh<obj,T>::make_anisotropic(const T & para_tau, const int & cas) const
   \param Th mesh \n
   \return 1 if equal, 0 if not
 */
+
+
 //==========================================================================
-template <template <typename U> class obj,class T>
-int operator ==(const mesh<obj,T>& Sh, const mesh<obj,T>&Th)
+template <class Fobj, class FT>
+int operator ==(const mesh<Fobj,FT>& Sh, const mesh<Fobj,FT>&Th)
 //==========================================================================
 {
   int boolean=(Sh.alive==Th.alive)&&(Sh.dimspace_==Th.dimspace_);
@@ -540,7 +532,9 @@ int operator ==(const mesh<obj,T>& Sh, const mesh<obj,T>&Th)
   \return A graphical windows containing the mesh drawn by gnuplot
 */
 //==========================================================================
-template <template <typename U> class obj,class T>
+//template <template <class> class obj, class T>
+//template <template <typename U> class obj,class T>
+template <class obj, class T>
 void mesh<obj,T>::visu2d() const
 //==========================================================================
 {
@@ -559,7 +553,9 @@ file1<<endl<<endl;
     }
 file1.close(); 
         
-ofstream file2("mesh.gnu"); 
+ofstream file2("mesh.gnu");
+//file2 << "set terminal postscript" << endl;
+//file2 << "set output \" maillage.ps \" " << endl;
 file2 << "plot \"mesh.dat\" with linespoints" << endl;
 file2 << "pause -1 \"Appuyer sur Entre\" " << endl; 
 file2.close(); 
