@@ -2,7 +2,7 @@
  *                 Finite Element Method Object Library        *
  *           class mesh_tri_2d : declaration for mesh_tri_2d   *
  *                    simula+@metz.ensam.fr                    *
- *	             GNU/linux version 0.1.0	               *
+ *	             GNU/linux version 0.3.0                   *
  *            software under General Public License            *
  ***************************************************************
  * copyright  2005,2006 CREUSE Emmanuel
@@ -45,7 +45,7 @@
 	     copyright \htmlonly &#169; \endhtmlonly  2006 BOULAAJINE Lahcen \n
 	     copyright \htmlonly &#169; \endhtmlonly Laboratoire de Physique et Mcanique des Matriaux (LPMM - UMR 7554) \n
 	     copyright \htmlonly &#169; \endhtmlonly Laboratoire de Mathmatiques et ses Applications de Valenciennes (LAMAV - EA 4015)
-    \version 0.2.4
+    \version 0.3.0
     \date 2002-2006
     \bug none
     \warning none
@@ -83,12 +83,24 @@
 using namespace std;
 
 //=============================
-template <class T> class mesh_tri_2d:public mesh<triangle,T>
+template <class T>
+class mesh_tri_2d:public mesh<triangle<T>,T>
 //=============================
 
 {
+  
+
   //protected :
   protected :
+
+  using mesh<triangle<T>,T>::alive;
+  using mesh<triangle<T>,T>::dimspace_;
+  using mesh<triangle<T>,T>::nbvertex_;
+  using mesh<triangle<T>,T>::nbelement_;
+  using mesh<triangle<T>,T>::vertices;
+  using mesh<triangle<T>,T>::element;
+
+
   int nbsegments_;
   matrix<int> SetOfEdges_;
   vector<int> Type_of_edge_;
@@ -105,7 +117,6 @@ template <class T> class mesh_tri_2d:public mesh<triangle,T>
   void remplir_descripteurs_anisotropes();
   vector<T> NormEdge_;
   matrix<T> NormalUnitVector_;
-  // Nadir et Manu 240106
   int NbMaxSegmentsPerNode_;
   vector<int> NbSegmentsPerNode_;
   public :
@@ -134,9 +145,12 @@ template <class T> class mesh_tri_2d:public mesh<triangle,T>
   matrix<T> NormalUnitVector()const{return(NormalUnitVector_);};
   int NbMaxSegmentsPerNode()const{return(NbMaxSegmentsPerNode_);};
   vector<int> NbSegmentsPerNode()const{return(NbSegmentsPerNode_);};
-  friend  int operator == <T> (const mesh_tri_2d<T>&,const mesh_tri_2d<T>&);
-  friend  int operator != <T> (const mesh_tri_2d<T>&,const mesh_tri_2d<T>&);
+  template <class FT> friend  int operator ==  (const mesh_tri_2d<FT>&,const mesh_tri_2d<FT>&);
+  template <class FT> friend  int operator !=  (const mesh_tri_2d<FT>&,const mesh_tri_2d<FT>&);
   int num_loc(int, int) const;
+  void internal_boundary_marking();
+  void internal_boundary_marking_lshape();
+  void internal_boundary_marking_reglshape();
 };
 
 
@@ -145,7 +159,7 @@ template <class T> class mesh_tri_2d:public mesh<triangle,T>
 */
 //=============================
 template <class T>
-mesh_tri_2d<T>::mesh_tri_2d() :mesh<triangle,T>()
+mesh_tri_2d<T>::mesh_tri_2d() :mesh<triangle<T>,T>()
 //=============================
 {
 nbsegments_=0;
@@ -153,7 +167,7 @@ nbsegments_=0;
 
 //=============================
 template <class T>
-mesh_tri_2d<T>::mesh_tri_2d(const char*file1, const char*file2) :mesh<triangle,T>(file1,file2)
+mesh_tri_2d<T>::mesh_tri_2d(const char*file1, const char*file2) :mesh<triangle<T>,T>(file1,file2)
 //=============================
 {
 
@@ -265,17 +279,10 @@ for (int jt=1; jt<=nbelement_; jt++)
 
   for (int i=1; i<=nbsegments_;i++)
   {
-    //cout <<"-----------------------------------------------\n";
-    //cout << "nsegment= " << i << "\n";
     NumMaille1=TrianglesPerEdge_(i,1); // c'est K+
     NumMaille2=TrianglesPerEdge_(i,2); // c'est K-
-    //cout << "NumMaille1= " << NumMaille1 << "\n";
-    //cout << "NumMaille2= " << NumMaille2 << "\n";
-    
     NumNoeudA=SetOfEdges_(i,1);
     NumNoeudB=SetOfEdges_(i,2);
-    //cout << "NumNoeudA= " << NumNoeudA << "\n";
-    //cout << "NumNoeudB= " << NumNoeudB << "\n";
 
     
     if (NumMaille1!=0)
@@ -283,9 +290,6 @@ for (int jt=1; jt<=nbelement_; jt++)
       NumLocNodeInTriangle_(i,1)=num_loc(NumMaille1,NumNoeudA);
       NumLocNodeInTriangle_(i,2)=num_loc(NumMaille1,NumNoeudB);
       NumLocSegInTriangle1=6-NumLocNodeInTriangle_(i,1)-NumLocNodeInTriangle_(i,2);
-      //cout << "NumLocNodeInTriangle_(" << i << ",1)= " << NumLocNodeInTriangle_(i,1)<<"\n";
-      //cout << "NumLocNodeInTriangle_(" << i << ",2)= " << NumLocNodeInTriangle_(i,2)<<"\n";
-     
      }
 
      if (NumMaille2!=0)
@@ -293,21 +297,14 @@ for (int jt=1; jt<=nbelement_; jt++)
       NumLocNodeInTriangle_(i,3)=num_loc(NumMaille2,NumNoeudA);
       NumLocNodeInTriangle_(i,4)=num_loc(NumMaille2,NumNoeudB);
       NumLocSegInTriangle2=6-NumLocNodeInTriangle_(i,3)-NumLocNodeInTriangle_(i,4);
-      //cout << "NumLocNodeInTriangle_(" << i << ",3)= " << NumLocNodeInTriangle_(i,3)<<"\n";
-      //cout << "NumLocNodeInTriangle_(" << i << ",4)= " << NumLocNodeInTriangle_(i,4)<<"\n";
-     
      }
 
     if ((NumMaille1!=0)&&(NumMaille2!=0))
      {
       NumberOfNeighboorOfTriangles_[NumMaille2]++;
-//      TrianglesPerTriangle_(NumMaille2,NumberOfNeighboorOfTriangles_[NumMaille2])=NumMaille1;
       TrianglesPerTriangle_(NumMaille2,NumLocSegInTriangle2)=NumMaille1;
-      //cout << "TrianglesPerTriangle_(" << NumMaille2 << "," << NumLocSegInTriangle2 << ")= " << NumMaille1 << "\n";
       NumberOfNeighboorOfTriangles_[NumMaille1]++;
-//      TrianglesPerTriangle_(NumMaille1,NumberOfNeighboorOfTriangles_[NumMaille1])=NumMaille2;
       TrianglesPerTriangle_(NumMaille1,NumLocSegInTriangle1)=NumMaille2;
-      //cout << "TrianglesPerTriangle_(" << NumMaille1 << "," << NumLocSegInTriangle1 << ")= " << NumMaille2 << "\n";
      }
   }
      hminT_=vector<T>(nbelement_);
@@ -333,7 +330,7 @@ for (int jt=1; jt<=nbelement_; jt++)
 
 //=============================
 template <class T>
-mesh_tri_2d<T>::mesh_tri_2d(const char*file1) :mesh<triangle,T>(file1)
+mesh_tri_2d<T>::mesh_tri_2d(const char*file1) :mesh<triangle<T>,T>(file1)
 //=============================
 {
 EdgesPerTriangle_=matrix<int>(nbelement_,3);
@@ -444,27 +441,15 @@ for (int jt=1; jt<=nbelement_; jt++)
 
   for (int i=1; i<=nbsegments_;i++)
   {
-    //cout <<"-----------------------------------------------\n";
-    //cout << "nsegment= " << i << "\n";
     NumMaille1=TrianglesPerEdge_(i,1); // c'est K+
     NumMaille2=TrianglesPerEdge_(i,2); // c'est K-
-    //cout << "NumMaille1= " << NumMaille1 << "\n";
-    //cout << "NumMaille2= " << NumMaille2 << "\n";
-    
     NumNoeudA=SetOfEdges_(i,1);
     NumNoeudB=SetOfEdges_(i,2);
-    //cout << "NumNoeudA= " << NumNoeudA << "\n";
-    //cout << "NumNoeudB= " << NumNoeudB << "\n";
-
-    
     if (NumMaille1!=0)
      {
       NumLocNodeInTriangle_(i,1)=num_loc(NumMaille1,NumNoeudA);
       NumLocNodeInTriangle_(i,2)=num_loc(NumMaille1,NumNoeudB);
       NumLocSegInTriangle1=6-NumLocNodeInTriangle_(i,1)-NumLocNodeInTriangle_(i,2);
-      //cout << "NumLocNodeInTriangle_(" << i << ",1)= " << NumLocNodeInTriangle_(i,1)<<"\n";
-      //cout << "NumLocNodeInTriangle_(" << i << ",2)= " << NumLocNodeInTriangle_(i,2)<<"\n";
-     
      }
 
      if (NumMaille2!=0)
@@ -472,21 +457,14 @@ for (int jt=1; jt<=nbelement_; jt++)
       NumLocNodeInTriangle_(i,3)=num_loc(NumMaille2,NumNoeudA);
       NumLocNodeInTriangle_(i,4)=num_loc(NumMaille2,NumNoeudB);
       NumLocSegInTriangle2=6-NumLocNodeInTriangle_(i,3)-NumLocNodeInTriangle_(i,4);
-      //cout << "NumLocNodeInTriangle_(" << i << ",3)= " << NumLocNodeInTriangle_(i,3)<<"\n";
-      //cout << "NumLocNodeInTriangle_(" << i << ",4)= " << NumLocNodeInTriangle_(i,4)<<"\n";
-     
      }
 
     if ((NumMaille1!=0)&&(NumMaille2!=0))
      {
       NumberOfNeighboorOfTriangles_[NumMaille2]++;
-//      TrianglesPerTriangle_(NumMaille2,NumberOfNeighboorOfTriangles_[NumMaille2])=NumMaille1;
       TrianglesPerTriangle_(NumMaille2,NumLocSegInTriangle2)=NumMaille1;
-      //cout << "TrianglesPerTriangle_(" << NumMaille2 << "," << NumLocSegInTriangle2 << ")= " << NumMaille1 << "\n";
       NumberOfNeighboorOfTriangles_[NumMaille1]++;
-//      TrianglesPerTriangle_(NumMaille1,NumberOfNeighboorOfTriangles_[NumMaille1])=NumMaille2;
       TrianglesPerTriangle_(NumMaille1,NumLocSegInTriangle1)=NumMaille2;
-      //cout << "TrianglesPerTriangle_(" << NumMaille1 << "," << NumLocSegInTriangle1 << ")= " << NumMaille2 << "\n";
      }
   }
      hminT_=vector<T>(nbelement_);
@@ -514,7 +492,8 @@ for (int jt=1; jt<=nbelement_; jt++)
 
 //=============================
 template <class T>
-mesh_tri_2d<T>::mesh_tri_2d(const mesh_tri_2d<T>& EF) :mesh<triangle,T>(EF)
+//mesh_tri_2d<T>::mesh_tri_2d(const mesh_tri_2d<T>& EF) :mesh<triangle,T>(EF)
+mesh_tri_2d<T>::mesh_tri_2d(const mesh_tri_2d<T>& EF) :mesh<triangle<T>,T>(EF)
 //=============================
 {
   nbsegments_=EF.nbsegments_;
@@ -541,14 +520,11 @@ template <class T>
 mesh_tri_2d<T>:: ~mesh_tri_2d()
 //=============================
 {
-  //    mesh<triangle,T>((*this)).~mesh<triangle,T>();
-  // (*this).~mesh<triangle,T>();
     SetOfEdges_.~matrix<int>();
     Type_of_edge_.~vector<int>();
     EdgesPerTriangle_.~matrix<int>();
     NbTrianglesPerEdge_.~vector<int>();
     TrianglesPerEdge_.~matrix<int>();
-    //NbTrianglesPerTriangle_.~vector<int>();
     TrianglesPerTriangle_.~matrix<int>();
     NumberOfNeighboorOfTriangles_.~vector<int>();
     NumLocNodeInTriangle_.~matrix<int>();
@@ -568,7 +544,7 @@ template <class T>
 mesh_tri_2d<T>& mesh_tri_2d<T>::operator =(const mesh_tri_2d<T>& EF)
 //========================================================
 {
- (*this).mesh<triangle,T>::operator=(EF);
+ (*this).mesh<triangle<T>,T>::operator=(EF);
   nbsegments_=EF.nbsegments_;
   SetOfEdges_=EF.SetOfEdges_;
   Type_of_edge_=EF.Type_of_edge_;
@@ -590,12 +566,12 @@ mesh_tri_2d<T>& mesh_tri_2d<T>::operator =(const mesh_tri_2d<T>& EF)
 }
 
 //==========================================================================
-template <class T>
-int operator ==(const mesh_tri_2d<T>& EF1,const mesh_tri_2d<T>& EF2)
+template <class FT>
+int operator ==(const mesh_tri_2d<FT>& EF1,const mesh_tri_2d<FT>& EF2)
 //==========================================================================
 {
   int boolean=1;
-  boolean*=(mesh<triangle,T>(EF1)==mesh<triangle,T>(EF2));
+  boolean*=(mesh<triangle<FT>,FT>(EF1)==mesh<triangle<FT>,FT>(EF2));
   boolean*=(EF1.nbsegments_==EF2.nbsegments_);
   boolean*=(EF1.SetOfEdges_==EF2.SetOfEdges_);
   boolean*=(EF1.Type_of_edge_==EF2.Type_of_edge_);
@@ -617,8 +593,8 @@ int operator ==(const mesh_tri_2d<T>& EF1,const mesh_tri_2d<T>& EF2)
 }
 
 //==========================================================================
-template <class T>
-int operator !=(const mesh_tri_2d<T>& EF1,const mesh_tri_2d<T>& EF2)
+template <class FT>
+int operator !=(const mesh_tri_2d<FT>& EF1,const mesh_tri_2d<FT>& EF2)
 //==========================================================================
 {
   return(!(EF1==EF2));
@@ -682,7 +658,6 @@ for (int i=1; i<=nbelement_; i++)
     hminT_[i]=deuxfoissurf/basemax;
   }
 
-//hminE_=vector<T>(nbsegments());
 for (int i=1; i<=nbelement_; i++)
   {
    for (int j=1; j<=3; j++)
@@ -790,7 +765,11 @@ for (int jt=1; jt<=nbelement_; jt++)
 //---------------------------------------------------------------------------------
 node<T>* vertices_new;
 triangle<T>* element_new;
-vertices_new=new node<T>[nbvertex_new](2);
+
+vertices_new = new node<T>[nbvertex_new];
+  for (int i=0; i<nbvertex_new; i++)
+       vertices_new[i] = node<T>(2);
+
 element_new=new triangle<T>[nbelement_new];
 
 node<T> ** nodes_of_element;
@@ -849,8 +828,6 @@ for (int jt=1; jt<=nbelement_; jt++)
 // Puis chacune des trois autres, que l'on stocke en queue de tableau
     for (int j=1; j<=3; j++)
        {
-         //cout << "jt= "  << jt << "number= " << &(element_new[jt-1][j])-vertices_new << endl;
-         //nodes_of_element[0]=&vertices_new[number((*this)[jt][j])-1];
 	 nodes_of_element[0]=&vertices_new[&(element[jt-1][j])-vertices];
 	 nodes_of_element[1]=&vertices_new[TABSEG[EdgesPerTriangle_(jt,prec[j])]-1];
 	 nodes_of_element[2]=&vertices_new[TABSEG[EdgesPerTriangle_(jt,suiv[j])]-1];
@@ -875,19 +852,15 @@ for (int jt=1; jt<=nbelement_; jt++)
        vertices_new[num_current_vertex-1][1]=xnoeud;
        vertices_new[num_current_vertex-1][2]=ynoeud;
        vertices_new[num_current_vertex-1].boundary_condition()=typenoeud;
-       //cout << vertices_new[num_current_vertex-1] << "\n";
        TABSEG[EdgesPerTriangle_(jt,jloc)]=num_current_vertex;
        num_current_vertex++;
       }
  
 // Cration des deux mailles : la premire est stocke  la place de l'ancienne, la seconde en queue de tableau.
        nodes_of_element[0]=&vertices_new[TABSEG[EdgesPerTriangle_(jt,jloc)]-1];
-       //nodes_of_element[1]=&vertices_new[number((*this)[jt][jloc])-1];
-       //nodes_of_element[2]=&vertices_new[number((*this)[jt][prec[jloc]])-1];
        nodes_of_element[1]=&vertices_new[&(element[jt-1][jloc])-vertices];
        nodes_of_element[2]=&vertices_new[&(element[jt-1][prec[jloc]])-vertices];
        element_new[jt-1]=triangle<T>(nodes_of_element);
-       //nodes_of_element[2]=&vertices_new[number((*this)[jt][suiv[jloc]])-1];
        nodes_of_element[2]=&vertices_new[&(element[jt-1][suiv[jloc]])-vertices];
        element_new[num_current_triangle-1]=triangle<T>(nodes_of_element);
        num_current_triangle++;
@@ -901,7 +874,10 @@ for (int jt=1; jt<=nbelement_; jt++)
 
 nbvertex_=nbvertex_new;
 delete [] vertices;
-vertices=new node<T>[nbvertex_](dimspace_);
+
+vertices = new node<T>[nbvertex_];
+  for (int i=0; i<nbvertex_; i++)
+       vertices[i] = node<T>(2);
 
 for(int i=0;i<nbvertex_;i++)
       vertices[i]=vertices_new[i];
@@ -915,7 +891,6 @@ for(int i=0;i<nbelement_;i++)
 
     for (int j=0;j<3;j++)
       {
-	// Pour Nadir : le bug tait l !!!
         nodes_of_element[j]=&vertices[&(element_new[i][j+1])-vertices_new];
        }
     element[i]=triangle<T>(nodes_of_element);
@@ -1073,17 +1048,11 @@ for (int jt=1; jt<=nbelement_; jt++)
 
   for (int i=1; i<=nbsegments_;i++)
   {
-    //cout <<"-----------------------------------------------\n";
-    //cout << "nsegment= " << i << "\n";
     NumMaille1=TrianglesPerEdge_(i,1); // c'est K+
     NumMaille2=TrianglesPerEdge_(i,2); // c'est K-
-    //cout << "NumMaille1= " << NumMaille1 << "\n";
-    //cout << "NumMaille2= " << NumMaille2 << "\n";
     
     NumNoeudA=SetOfEdges_(i,1);
     NumNoeudB=SetOfEdges_(i,2);
-    //cout << "NumNoeudA= " << NumNoeudA << "\n";
-    //cout << "NumNoeudB= " << NumNoeudB << "\n";
 
     
     if (NumMaille1!=0)
@@ -1091,8 +1060,6 @@ for (int jt=1; jt<=nbelement_; jt++)
       NumLocNodeInTriangle_(i,1)=num_loc(NumMaille1,NumNoeudA);
       NumLocNodeInTriangle_(i,2)=num_loc(NumMaille1,NumNoeudB);
       NumLocSegInTriangle1=6-NumLocNodeInTriangle_(i,1)-NumLocNodeInTriangle_(i,2);
-      //cout << "NumLocNodeInTriangle_(" << i << ",1)= " << NumLocNodeInTriangle_(i,1)<<"\n";
-      //cout << "NumLocNodeInTriangle_(" << i << ",2)= " << NumLocNodeInTriangle_(i,2)<<"\n";
      
      }
 
@@ -1101,35 +1068,29 @@ for (int jt=1; jt<=nbelement_; jt++)
       NumLocNodeInTriangle_(i,3)=num_loc(NumMaille2,NumNoeudA);
       NumLocNodeInTriangle_(i,4)=num_loc(NumMaille2,NumNoeudB);
       NumLocSegInTriangle2=6-NumLocNodeInTriangle_(i,3)-NumLocNodeInTriangle_(i,4);
-      //cout << "NumLocNodeInTriangle_(" << i << ",3)= " << NumLocNodeInTriangle_(i,3)<<"\n";
-      //cout << "NumLocNodeInTriangle_(" << i << ",4)= " << NumLocNodeInTriangle_(i,4)<<"\n";
      
      }
 
     if ((NumMaille1!=0)&&(NumMaille2!=0))
      {
       NumberOfNeighboorOfTriangles_[NumMaille2]++;
-//      TrianglesPerTriangle_(NumMaille2,NumberOfNeighboorOfTriangles_[NumMaille2])=NumMaille1;
       TrianglesPerTriangle_(NumMaille2,NumLocSegInTriangle2)=NumMaille1;
-      //cout << "TrianglesPerTriangle_(" << NumMaille2 << "," << NumLocSegInTriangle2 << ")= " << NumMaille1 << "\n";
       NumberOfNeighboorOfTriangles_[NumMaille1]++;
-//      TrianglesPerTriangle_(NumMaille1,NumberOfNeighboorOfTriangles_[NumMaille1])=NumMaille2;
       TrianglesPerTriangle_(NumMaille1,NumLocSegInTriangle1)=NumMaille2;
-      //cout << "TrianglesPerTriangle_(" << NumMaille1 << "," << NumLocSegInTriangle1 << ")= " << NumMaille2 << "\n";
      }
   }
 
-     hminT_.~vector();
-     hE_.~vector();
-     hminE_.~vector();
+     hminT_.~vector<T>();
+     hE_.~vector<T>();
+     hminE_.~vector<T>();
      hminT_=vector<T>(nbelement_);
      hE_=vector<T>(nbsegments_);
      hminE_=vector<T>(nbsegments_);
 
      remplir_descripteurs_anisotropes();
 
-     NormEdge_.~vector();
-     NormalUnitVector_.~matrix();
+     NormEdge_.~vector<T>();
+     NormalUnitVector_.~matrix<T>();
 
      NormEdge_=vector<T>(nbsegments_);
      NormalUnitVector_=matrix<T>(nbsegments_,2);
@@ -1231,17 +1192,16 @@ for (int RESTART=1 ; RESTART ; )
 					 	TABTRI[numtr]++;
 						TABSEG[EdgesPerTriangle_(jt,prec[num])]=-1;
 						}
-					     //if ((TABTRI[numtr]>=2)&&(numtr<jt))
-					     if ((TABTRI[numtr]>=1)&&(numtr<jt)) // Modif Manu 250106
+					     if ((TABTRI[numtr]>=1)&&(numtr<jt)) 
 						 RESTART=1;
 					  }
-					  // Ajout Manu 210106
+				
 					 else
 					   if (TABSEG[EdgesPerTriangle_(jt,prec[num])]!=-1)
 					   {
 					    TABSEG[EdgesPerTriangle_(jt,prec[num])]=-1;
 					   }
-					 // Fin Ajout Manu 210106
+					
 
 					}
 			else if (alpha_suiv >= 2.0*MinAngle)
@@ -1259,17 +1219,17 @@ for (int RESTART=1 ; RESTART ; )
 					 	TABTRI[numtr]++;
 						TABSEG[EdgesPerTriangle_(jt,suiv[num])]=-1;
 						}
-					     //if ((TABTRI[numtr]>=2)&&(numtr<jt))
-					       if ((TABTRI[numtr]>=1)&&(numtr<jt)) // Modif Manu 250106  
+					    
+					       if ((TABTRI[numtr]>=1)&&(numtr<jt))   
 				                 RESTART=1;
 					  }
-					  // Ajout Manu 210106
+					
 					 else
 					   if (TABSEG[EdgesPerTriangle_(jt,suiv[num])]!=-1)
 					   {
 					    TABSEG[EdgesPerTriangle_(jt,suiv[num])]=-1;
 					   }
-					 // Fin Ajout Manu 210106
+					
 					}
 			// Ajout Manu 210106 : Aucune des 3 possibilits ne permet d'avoir
 			// la condition de l'angle minimum respecte... Dans ce cas il faut couper la maille en quatre !!!
@@ -1285,8 +1245,8 @@ for (int RESTART=1 ; RESTART ; )
 					    TABTRI[numtr]++;
        					    TABSEG[EdgesPerTriangle_(jt,j)]=-1;
 					 }
-					//if ((TABTRI[numtr]>=2)&&(numtr<jt))
-					if ((TABTRI[numtr]>=1)&&(numtr<jt)) // Modif Manu 250106
+				
+					if ((TABTRI[numtr]>=1)&&(numtr<jt)) 
 				                 RESTART=1;
 					 }
 					 else
@@ -1296,7 +1256,7 @@ for (int RESTART=1 ; RESTART ; )
 					   }
      				}
 		            }
-			// Fin Ajout Manu 210106
+			
            
      	       }
    else if (TABTRI[jt]==2)
@@ -1332,12 +1292,12 @@ for (int RESTART=1 ; RESTART ; )
 		 	for (int j=1; j<=3; j++)
                   		{
         			  if (TrianglesPerTriangle_(jt,j))
-				    //if (TABTRI[TrianglesPerTriangle_(jt,j)]!=-1) TABTRI[TrianglesPerTriangle_(jt,j)]++;
-                                   if (TABSEG[EdgesPerTriangle_(jt,j)]!=-1)  TABTRI[TrianglesPerTriangle_(jt,j)]++; // Modif Manu 250106
+				  
+                                   if (TABSEG[EdgesPerTriangle_(jt,j)]!=-1)  TABTRI[TrianglesPerTriangle_(jt,j)]++; 
        				  TABSEG[EdgesPerTriangle_(jt,j)]=-1;
         			  if (TrianglesPerTriangle_(jt,j))
-				    //if ((TABTRI[TrianglesPerTriangle_(jt,j)]>=2)&&(TrianglesPerTriangle_(jt,j)<jt)) RESTART=1;
-				    if ((TABTRI[TrianglesPerTriangle_(jt,j)]>=1)&&(TrianglesPerTriangle_(jt,j)<jt)) RESTART=1; // Modif Manu 250106
+			
+				    if ((TABTRI[TrianglesPerTriangle_(jt,j)]>=1)&&(TrianglesPerTriangle_(jt,j)<jt)) RESTART=1; 
       		  		}
 			}
 	                else  TABTRI[jt]=-2;
@@ -1351,20 +1311,17 @@ for (int RESTART=1 ; RESTART ; )
 		           for (int j=1; j<=3; j++)
                      		{
         			  if (TrianglesPerTriangle_(jt,j))
-       				  //if (TABTRI[TrianglesPerTriangle_(jt,j)]!=-1 && TABTRI[TrianglesPerTriangle_(jt,j)]!=-2
-				  // Modif Manu 240106 :
 				   if (TABTRI[TrianglesPerTriangle_(jt,j)]!=-1 && TABSEG[EdgesPerTriangle_(jt,j)]!=-1)
 				   TABTRI[TrianglesPerTriangle_(jt,j)]++;
 				  
        				  TABSEG[EdgesPerTriangle_(jt,j)]=-1;
 				  
         			  if (TrianglesPerTriangle_(jt,j))
-				    //if ((TABTRI[TrianglesPerTriangle_(jt,j)]>=2)&&(TrianglesPerTriangle_(jt,j)<jt)) RESTART=1;
-				    if ((TABTRI[TrianglesPerTriangle_(jt,j)]>=1)&&(TrianglesPerTriangle_(jt,j)<jt)) RESTART=1; // Modif Manu 250106
+				    if ((TABTRI[TrianglesPerTriangle_(jt,j)]>=1)&&(TrianglesPerTriangle_(jt,j)<jt)) RESTART=1; 
       		     		}
 
 		}
-		//Ajout Manu 230106
+	
     else if (TABTRI[jt]==-1)
     {
      for (int j=1; j<=3; j++)
@@ -1372,7 +1329,7 @@ for (int RESTART=1 ; RESTART ; )
        				  TABSEG[EdgesPerTriangle_(jt,j)]=-1;
       		     		}
     }
-    // Fin Ajout Manu 230106
+
 
   //=============================================================
   //=================Fin Ammelioration_1=========================
@@ -1391,16 +1348,14 @@ TABSEGLOC=TABSEG;
 
 for (int jt=1; jt<=nbelement_; jt++)
 {
-//cout << "----------------------------------------------";
-//cout << "jt= " << jt << endl;
-//cout << "TABTRI[" << jt << "]= " <<  TABTRI[jt] << endl;
+
 
  if (TABTRI[jt]==-1)
   {
    nbelement_new+=3;
    for (int j=1; j<=3; j++)
     {
-  //   cout << "TABSEGLOC[" << EdgesPerTriangle_(jt,j) << "]= " << TABSEGLOC[EdgesPerTriangle_(jt,j)] << endl;
+
      if (TABSEGLOC[EdgesPerTriangle_(jt,j)]==-1)
      {
       nbvertex_new++;
@@ -1415,7 +1370,7 @@ for (int jt=1; jt<=nbelement_; jt++)
     nbelement_new+=2;
     for (int j=1; j<=3; j++)
     {
-    // cout << "TABSEGLOC[" << EdgesPerTriangle_(jt,j) << "]= " << TABSEGLOC[EdgesPerTriangle_(jt,j)] << endl;
+ 
      if (TABSEGLOC[EdgesPerTriangle_(jt,j)]==-1)
      {
       nbvertex_new++;
@@ -1430,7 +1385,6 @@ for (int jt=1; jt<=nbelement_; jt++)
     nbelement_new++;
     for (int j=1; j<=3; j++)
     {
-     //cout << "TABSEGLOC[" << EdgesPerTriangle_(jt,j) << "]= " << TABSEGLOC[EdgesPerTriangle_(jt,j)] << endl;
      if (TABSEGLOC[EdgesPerTriangle_(jt,j)]==-1)
      {
       nbvertex_new++;
@@ -1438,18 +1392,19 @@ for (int jt=1; jt<=nbelement_; jt++)
      }
     }
    }
-   //cout << "nbvertex_new= " << nbvertex_new << endl;
-   //cout << "---------------------------------------";
   }
-
-//cout << "Avant la phase 3, nbvertex_new= " << nbvertex_new << " nbelement_new= " << nbelement_new << endl;
 //---------------------------------------------------------------------------------
 // Phase 3 : Cration et remplissage des tableaux contenant le nouveau maillage //
 //---------------------------------------------------------------------------------
 
 node<T>* vertices_new;
 triangle<T>* element_new;
-vertices_new=new node<T>[nbvertex_new](2);
+
+vertices_new = new node<T>[nbvertex_new];
+  for (int i=0; i<nbvertex_new; i++)
+       vertices_new[i] = node<T>(2);
+
+
 element_new=new triangle<T>[nbelement_new];
 
 node<T> ** nodes_of_element;
@@ -1476,8 +1431,7 @@ int jloc;
 
 for (int jt=1; jt<=nbelement_; jt++)
 {
-  //cout << "------------------------------------------------------------";
-  //cout << "jt= " << jt << "TABTRI[" << jt << "]= " << TABTRI[jt] << endl;
+
  if (TABTRI[jt]==-1)
   {
    for (int j=1; j<=3; j++)
@@ -1500,24 +1454,20 @@ for (int jt=1; jt<=nbelement_; jt++)
 
     for (int j=0; j<=2; j++)
         {
-	//cout << "j= " << j << endl;
-	//cout << "EdgesPerTriangle_(jt,j+1)= " << EdgesPerTriangle_(jt,j+1) << endl;
-	//cout << "TABSEG[EdgesPerTriangle_(jt,j+1)]= " << TABSEG[EdgesPerTriangle_(jt,j+1)] << endl;
         nodes_of_element[j]=&vertices_new[TABSEG[EdgesPerTriangle_(jt,j+1)]-1];
 	}
         element_new[jt-1]=triangle<T>(nodes_of_element);
-	//cout << "nouvelle maille jt= " << jt << "maille : " << element_new[jt-1] << endl;
+	
 // Puis chacune des trois autres, que l'on stocke en queue de tableau
     for (int j=1; j<=3; j++)
        {
-         //cout << "jt= "  << jt << "number= " << &(element_new[jt-1][j])-vertices_new << endl;
-         //nodes_of_element[0]=&vertices_new[number((*this)[jt][j])-1];
+
 	 nodes_of_element[0]=&vertices_new[&(element[jt-1][j])-vertices];
 	 nodes_of_element[1]=&vertices_new[TABSEG[EdgesPerTriangle_(jt,prec[j])]-1];
 	 nodes_of_element[2]=&vertices_new[TABSEG[EdgesPerTriangle_(jt,suiv[j])]-1];
 
 	 element_new[num_current_triangle-1]=triangle<T>(nodes_of_element);
-	 //cout << "nouvelle maille jt= " <<  num_current_triangle << "maille : " << element_new[num_current_triangle-1] << endl;
+	
 	 num_current_triangle++;
        }
       }
@@ -1537,7 +1487,7 @@ for (int jt=1; jt<=nbelement_; jt++)
      			double alpha_prec, alpha_suiv;
      			alpha_prec = acos(-(u*w)/(u.norm()*w.norm()));
      			alpha_suiv = acos((v*w)/(v.norm()*w.norm()));
-     		//cout << "alpha_prec= "<< alpha_prec<< "alpha_suiv= "<< alpha_suiv<<"\n";
+     	
 // Cration ventuelle du 1er noeud
    if (TABSEG[EdgesPerTriangle_(jt,suiv[jloc])]==-1)
       {
@@ -1547,7 +1497,7 @@ for (int jt=1; jt<=nbelement_; jt++)
        vertices_new[num_current_vertex-1][1]=xnoeud;
        vertices_new[num_current_vertex-1][2]=ynoeud;
        vertices_new[num_current_vertex-1].boundary_condition()=typenoeud;
-      // cout <<"1er"<< vertices_new[num_current_vertex-1] << "\n";
+   
        TABSEG[EdgesPerTriangle_(jt,suiv[jloc])]=num_current_vertex;
        num_current_vertex++;
       }
@@ -1562,7 +1512,7 @@ for (int jt=1; jt<=nbelement_; jt++)
        vertices_new[num_current_vertex-1][1]=xnoeud;
        vertices_new[num_current_vertex-1][2]=ynoeud;
        vertices_new[num_current_vertex-1].boundary_condition()=typenoeud;
-       //cout <<"2emme"<< vertices_new[num_current_vertex-1] << "\n";
+     
        TABSEG[EdgesPerTriangle_(jt,prec[jloc])]=num_current_vertex;
        num_current_vertex++;
       }
@@ -1573,24 +1523,22 @@ for (int jt=1; jt<=nbelement_; jt++)
        nodes_of_element[1]=&vertices_new[TABSEG[EdgesPerTriangle_(jt,prec[jloc])]-1];
        nodes_of_element[2]=&vertices_new[TABSEG[EdgesPerTriangle_(jt,suiv[jloc])]-1];
        element_new[jt-1]=triangle<T>(nodes_of_element);
-       //cout << "nouvelle maille jt= " << jt << "maille : " << element_new[jt-1] << endl;
-       //nodes_of_element[1]=&vertices_new[number((*this)[jt][jloc])-1];
-       //nodes_of_element[2]=&vertices_new[number((*this)[jt][prec[jloc]])-1];
+    
        if (alpha_prec >= 2.0*MinAngle)
        {
        nodes_of_element[0]=&vertices_new[TABSEG[EdgesPerTriangle_(jt,prec[jloc])]-1];
        nodes_of_element[1]=&vertices_new[&(element[jt-1][suiv[jloc]])-vertices];
        nodes_of_element[2]=&vertices_new[&(element[jt-1][prec[jloc]])-vertices];
        element_new[num_current_triangle-1]=triangle<T>(nodes_of_element);
-       //cout << "nouvelle maille jt= " << num_current_triangle << "maille : " << element_new[num_current_triangle-1] << endl;
+
        num_current_triangle++;
        nodes_of_element[0]=&vertices_new[TABSEG[EdgesPerTriangle_(jt,prec[jloc])]-1];
        nodes_of_element[1]=&vertices_new[&(element[jt-1][prec[jloc]])-vertices];
-       // ligne modifie EC 180106 (il y avait une erreur : nodes_of_element[0] a la place de nodes_of_element[2])
+
        nodes_of_element[2]=&vertices_new[TABSEG[EdgesPerTriangle_(jt,suiv[jloc])]-1];
-       // fin de ligne modifie
+
        element_new[num_current_triangle-1]=triangle<T>(nodes_of_element);
-       //cout << "nouvelle maille jt= " << num_current_triangle << "maille : " << element_new[num_current_triangle-1] << endl;
+
        num_current_triangle++;
 
        }
@@ -1600,15 +1548,15 @@ for (int jt=1; jt<=nbelement_; jt++)
        nodes_of_element[1]=&vertices_new[&(element[jt-1][suiv[jloc]])-vertices];
        nodes_of_element[2]=&vertices_new[TABSEG[EdgesPerTriangle_(jt,suiv[jloc])]-1];
        element_new[num_current_triangle-1]=triangle<T>(nodes_of_element);
-       //cout << "nouvelle maille jt= " << num_current_triangle << "maille : " << element_new[num_current_triangle-1] << endl;
+     
        num_current_triangle++;
        nodes_of_element[0]=&vertices_new[TABSEG[EdgesPerTriangle_(jt,suiv[jloc])]-1];
        nodes_of_element[1]=&vertices_new[&(element[jt-1][suiv[jloc]])-vertices];
-       // ligne modifie EC 180106 (il y avait une erreur : nodes_of_element[0] a la place de nodes_of_element[2])
+
        nodes_of_element[2]=&vertices_new[&(element[jt-1][prec[jloc]])-vertices];
-       // fin de ligne modifie
+ 
        element_new[num_current_triangle-1]=triangle<T>(nodes_of_element);
-       //cout << "nouvelle maille jt= " << num_current_triangle << "maille : " << element_new[num_current_triangle-1] << endl;
+
        num_current_triangle++;
 
        }
@@ -1630,25 +1578,23 @@ for (int jt=1; jt<=nbelement_; jt++)
        vertices_new[num_current_vertex-1][1]=xnoeud;
        vertices_new[num_current_vertex-1][2]=ynoeud;
        vertices_new[num_current_vertex-1].boundary_condition()=typenoeud;
-       ////cout << vertices_new[num_current_vertex-1] << "\n";
+  
        TABSEG[EdgesPerTriangle_(jt,jloc)]=num_current_vertex;
        num_current_vertex++;
       }
 
 // Cration des deux mailles : la premire est stocke  la place de l'ancienne, la seconde en queue de tableau.
        nodes_of_element[0]=&vertices_new[TABSEG[EdgesPerTriangle_(jt,jloc)]-1];
-       //nodes_of_element[1]=&vertices_new[number((*this)[jt][jloc])-1];
-       //nodes_of_element[2]=&vertices_new[number((*this)[jt][prec[jloc]])-1];
+
        nodes_of_element[1]=&vertices_new[&(element[jt-1][jloc])-vertices];
        nodes_of_element[2]=&vertices_new[&(element[jt-1][prec[jloc]])-vertices];
        element_new[jt-1]=triangle<T>(nodes_of_element);
-       //cout << "nouvelle maille jt= " << jt << "maille : " << element_new[jt-1] << endl;
-       //nodes_of_element[2]=&vertices_new[number((*this)[jt][suiv[jloc]])-1];
+
        nodes_of_element[2]=&vertices_new[&(element[jt-1][suiv[jloc]])-vertices];
        element_new[num_current_triangle-1]=triangle<T>(nodes_of_element);
-       //cout << "nouvelle maille jt= " << num_current_triangle << "maille : " << element_new[num_current_triangle-1] << endl;
+
        num_current_triangle++;
- // cout<<TABSEG[EdgesPerTriangle_(jt,1)]<<" " <<  TABSEG[EdgesPerTriangle_(jt,2)] <<" " << TABSEG[EdgesPerTriangle_(jt,3)]  <<"\n";
+
   }
 
 }
@@ -1661,7 +1607,10 @@ for (int jt=1; jt<=nbelement_; jt++)
 
 nbvertex_=nbvertex_new;
 delete [] vertices;
-vertices=new node<T>[nbvertex_](dimspace_);
+
+  vertices = new node<T>[nbvertex_];
+  for (int i=0; i<nbvertex_; i++)
+       vertices[i] = node<T>(2);
 
 for(int i=0;i<nbvertex_;i++)
       vertices[i]=vertices_new[i];
@@ -1677,14 +1626,14 @@ for(int i=0;i<nbelement_;i++)
     for (int j=0;j<3;j++)
       {
 
-	// Pour Nadir : le bug tait l !!!
+	
         nodes_of_element[j]=&vertices[&(element_new[i][j+1])-vertices_new];
        }
     element[i]=triangle<T>(nodes_of_element);
 
   }
 delete[] nodes_of_element;
-//cout << "Fin phase 4" << endl;
+
 
 // ---------------------------------------------------------------------------------------------------------
 // Phase 5 : gnration du fichier de sortie pour sauvegarde du nouveau maillage (par defaut, non effectu)
@@ -1734,31 +1683,28 @@ VertexOfEdge(1,1)=2;VertexOfEdge(1,2)=3;
 VertexOfEdge(2,1)=3;VertexOfEdge(2,2)=1;
 VertexOfEdge(3,1)=1;VertexOfEdge(3,2)=2;
 
-//cout << "coucou 1 " << endl;
+
 
 int tmp,p;
 nbsegments_=0;
 vector<int> head(nbvertex_);
-//cout << "coucou 2 " << endl;
+
 int nbe=nbvertex_+nbelement_;
 vector<int> next(nbe);
 matrix<int> HypSetOfedges(nbe,2);
 vector<int> HypType_of_edge_(nbe);
 for(int i=1;i<=nbe;i++)
 HypType_of_edge_[i]=1;
-//cout << "coucou 3 " << endl;
+
 
 for(int k=1;k<=nbelement_;k++)
-{   //    cout << "---------------------------------" << endl ;
-        //cout << "k= " << k << endl;
+{   
 	for(int edge_number=1;edge_number<=3;edge_number++)
 	{
-	//cout << "edge_number= " << edge_number << endl;
-	//cout << "VertexOfEdge(edge_number,1)= " << VertexOfEdge(edge_number,1) << endl;
-	//cout << "VertexOfEdge(edge_number,2)= " << VertexOfEdge(edge_number,2) << endl;
+	
 	int i=number(element[k-1][VertexOfEdge(edge_number,1)]);
 	int j=number(element[k-1][VertexOfEdge(edge_number,2)]);
-	//cout << "i= " << i << " j= " << j << endl;
+	
 	if(j<i)
 		{tmp=i;i=j;j=tmp;}
 	int exists=0;
@@ -1786,14 +1732,14 @@ for(int k=1;k<=nbelement_;k++)
 		head[i]=EdgesPerTriangle_(k,p)=nbsegments_;
 	}
 }
-//cout << "fin k= " << k << endl;
+
 }
-//cout << "coucou 4 " << endl;
+
 SetOfEdges_.~matrix();
 SetOfEdges_=matrix<int>(nbsegments_,2);
 Type_of_edge_.~vector();
 Type_of_edge_=vector<int>(nbsegments_);
-//cout << "coucou 5 " << endl;
+
 for(int i=1;i<=nbsegments_;i++)
 {
 SetOfEdges_[i]=HypSetOfedges[i];
@@ -1809,7 +1755,7 @@ for (int numseg=1; numseg<=nbsegments_;numseg++)
  NbSegmentsPerNode_[SetOfEdges_(numseg,2)]++;
 } 
 NbMaxSegmentsPerNode_=NbSegmentsPerNode_.norminf();
-//cout << "Avant remplissage NbTrianglesPerEdge et al" << endl;
+
 //---------------------------------------------------------------------------------------------------
 // Remplissage de NbTrianglesPerEdge_ et de TrianglesPerEdge_
 // Remplissage de NormalDirection_ (29 Mars 2005 - Emmanuel Creus) : on rappelle que le jeme segment
@@ -1840,7 +1786,7 @@ for (int jt=1; jt<=nbelement_; jt++)
    TrianglesPerEdge_(numseg,NbTrianglesPerEdge_[numseg])=jt;
   }
 }
-//cout << "Fin remplissage NbTrianglesPerEdge et al" << endl;
+
 //---------------------------------------------------------------------------------------------------
 // Remplissage de  NumberOfNeighboorOfTriangles_,TrianglesPerTriangle_ et NumLocNodeInTriangle_
 //---------------------------------------------------------------------------------------------------
@@ -1858,17 +1804,14 @@ for (int jt=1; jt<=nbelement_; jt++)
 
   for (int i=1; i<=nbsegments_;i++)
   {
-    //cout <<"-----------------------------------------------\n";
-    //cout << "nsegment= " << i << "\n";
+   
     NumMaille1=TrianglesPerEdge_(i,1); // c'est K+
     NumMaille2=TrianglesPerEdge_(i,2); // c'est K-
-    //cout << "NumMaille1= " << NumMaille1 << "\n";
-    //cout << "NumMaille2= " << NumMaille2 << "\n";
+  
     
     NumNoeudA=SetOfEdges_(i,1);
     NumNoeudB=SetOfEdges_(i,2);
-    //cout << "NumNoeudA= " << NumNoeudA << "\n";
-    //cout << "NumNoeudB= " << NumNoeudB << "\n";
+  
 
     
     if (NumMaille1!=0)
@@ -1876,8 +1819,7 @@ for (int jt=1; jt<=nbelement_; jt++)
       NumLocNodeInTriangle_(i,1)=num_loc(NumMaille1,NumNoeudA);
       NumLocNodeInTriangle_(i,2)=num_loc(NumMaille1,NumNoeudB);
       NumLocSegInTriangle1=6-NumLocNodeInTriangle_(i,1)-NumLocNodeInTriangle_(i,2);
-      //cout << "NumLocNodeInTriangle_(" << i << ",1)= " << NumLocNodeInTriangle_(i,1)<<"\n";
-      //cout << "NumLocNodeInTriangle_(" << i << ",2)= " << NumLocNodeInTriangle_(i,2)<<"\n";
+     
      
      }
 
@@ -1886,35 +1828,33 @@ for (int jt=1; jt<=nbelement_; jt++)
       NumLocNodeInTriangle_(i,3)=num_loc(NumMaille2,NumNoeudA);
       NumLocNodeInTriangle_(i,4)=num_loc(NumMaille2,NumNoeudB);
       NumLocSegInTriangle2=6-NumLocNodeInTriangle_(i,3)-NumLocNodeInTriangle_(i,4);
-      //cout << "NumLocNodeInTriangle_(" << i << ",3)= " << NumLocNodeInTriangle_(i,3)<<"\n";
-      //cout << "NumLocNodeInTriangle_(" << i << ",4)= " << NumLocNodeInTriangle_(i,4)<<"\n";
+   
      
      }
 
     if ((NumMaille1!=0)&&(NumMaille2!=0))
      {
       NumberOfNeighboorOfTriangles_[NumMaille2]++;
-//      TrianglesPerTriangle_(NumMaille2,NumberOfNeighboorOfTriangles_[NumMaille2])=NumMaille1;
+
       TrianglesPerTriangle_(NumMaille2,NumLocSegInTriangle2)=NumMaille1;
-      //cout << "TrianglesPerTriangle_(" << NumMaille2 << "," << NumLocSegInTriangle2 << ")= " << NumMaille1 << "\n";
+
       NumberOfNeighboorOfTriangles_[NumMaille1]++;
-//      TrianglesPerTriangle_(NumMaille1,NumberOfNeighboorOfTriangles_[NumMaille1])=NumMaille2;
+
       TrianglesPerTriangle_(NumMaille1,NumLocSegInTriangle1)=NumMaille2;
-      //cout << "TrianglesPerTriangle_(" << NumMaille1 << "," << NumLocSegInTriangle1 << ")= " << NumMaille2 << "\n";
      }
   }
 
-     hminT_.~vector();
-     hE_.~vector();
-     hminE_.~vector();
+     hminT_.~vector<T>();
+     hE_.~vector<T>();
+     hminE_.~vector<T>();
      hminT_=vector<T>(nbelement_);
      hE_=vector<T>(nbsegments_);
      hminE_=vector<T>(nbsegments_);
 
      remplir_descripteurs_anisotropes();
 
-     NormEdge_.~vector();
-     NormalUnitVector_.~matrix();
+     NormEdge_.~vector<T>();
+     NormalUnitVector_.~matrix<T>();
 
      NormEdge_=vector<T>(nbsegments_);
      NormalUnitVector_=matrix<T>(nbsegments_,2);
@@ -1932,5 +1872,73 @@ for (int jt=1; jt<=nbelement_; jt++)
 
  }
 
+ //=============================
+template <class T>
+void mesh_tri_2d<T>::internal_boundary_marking()
+//=============================
+// This function set a 2 for the descriptor Type_of_edge_ if the edge is on the line x=1 as well as
+// for the nodes 
+{
+T xnoeud1,xnoeud2;
+for (int iseg=1; iseg<=nbsegments_;iseg++)
+  {
+  xnoeud1=vertices[SetOfEdges_(iseg,1)-1][1];
+  xnoeud2=vertices[SetOfEdges_(iseg,2)-1][1];
+  if ((abs(xnoeud1-1.0)<epsilon)&&(abs(xnoeud2-1.0)<epsilon))
+   {
+   Type_of_edge_[iseg]=2;
+   vertices[SetOfEdges_(iseg,1)-1].boundary_condition()=2;
+   vertices[SetOfEdges_(iseg,2)-1].boundary_condition()=2;
+   }
+  }
+}
+
+//=============================
+template <class T>
+void mesh_tri_2d<T>::internal_boundary_marking_lshape()
+//=============================
+// This function set a 2 for the descriptor Type_of_edge_ if the edge is on the line x=2 and
+// if y<=1, as well as for the nodes
+{
+T xnoeud1,xnoeud2;
+T ynoeud1,ynoeud2;
+for (int iseg=1; iseg<=nbsegments_;iseg++)
+  {
+  xnoeud1=vertices[SetOfEdges_(iseg,1)-1][1];
+  xnoeud2=vertices[SetOfEdges_(iseg,2)-1][1];
+  ynoeud1=vertices[SetOfEdges_(iseg,1)-1][2];
+  ynoeud2=vertices[SetOfEdges_(iseg,2)-1][2];
+  if ((abs(xnoeud1-2.0)<epsilon)&&(abs(xnoeud2-2.0)<epsilon)&&(ynoeud1<1.0+epsilon)&&(ynoeud2<1.0+epsilon))
+   {
+   Type_of_edge_[iseg]=2;
+   vertices[SetOfEdges_(iseg,1)-1].boundary_condition()=2;
+   vertices[SetOfEdges_(iseg,2)-1].boundary_condition()=2;
+   }
+  }
+}
+
+//=============================
+template <class T>
+void mesh_tri_2d<T>::internal_boundary_marking_reglshape()
+//=============================
+// This function set a 2 for the descriptor Type_of_edge_ if the edge is on the line x=0 and
+// if y>=0, as well as for the nodes
+{
+T xnoeud1,xnoeud2;
+T ynoeud1,ynoeud2;
+for (int iseg=1; iseg<=nbsegments_;iseg++)
+  {
+  xnoeud1=vertices[SetOfEdges_(iseg,1)-1][1];
+  xnoeud2=vertices[SetOfEdges_(iseg,2)-1][1];
+  ynoeud1=vertices[SetOfEdges_(iseg,1)-1][2];
+  ynoeud2=vertices[SetOfEdges_(iseg,2)-1][2];
+  if ((abs(xnoeud1)<epsilon)&&(abs(xnoeud2)<epsilon)&&(ynoeud1>-epsilon)&&(ynoeud2>-epsilon))
+   {
+   Type_of_edge_[iseg]=2;
+   vertices[SetOfEdges_(iseg,1)-1].boundary_condition()=2;
+   vertices[SetOfEdges_(iseg,2)-1].boundary_condition()=2;
+   }
+  }
+}
 #endif
 
